import pytest

from briefcase.config import AppConfig


@pytest.fixture
def first_app_config():
    return AppConfig(
        app_name="first-app",
        project_name="First Project",
        formal_name="First App",
        author="Megacorp",
        author_email="maintainer@example.com",
        url="https://example.com/first-app",
        bundle="com.example",
        version="0.0.1",
        description="The first simple app \\ demonstration",
        sources=["src/first_app"],
        requires=["foo==1.2.3", "bar>=4.5"],
        test_requires=["pytest"],
        license={"file": "LICENSE"},
    )


@pytest.fixture
def uppercase_app_config():
    return AppConfig(
        app_name="First-App",
        project_name="First Project",
        formal_name="First App",
        author="Megacorp",
        bundle="com.example",
        version="0.0.1",
        description="The first simple app",
        sources=["src/First_App"],
        license={"file": "LICENSE"},
    )


@pytest.fixture()
def underscore_app_config(first_app_config):
    return AppConfig(
        app_name="first_app",
        project_name="First Project",
        formal_name="First App",
        author="Megacorp",
        author_email="maintainer@example.com",
        url="https://example.com/first-app",
        bundle="com.example",
        version="0.0.1",
        description="The first simple app \\ demonstration",
        sources=["src/first_app"],
        license={"file": "LICENSE"},
        requires=["foo==1.2.3", "bar>=4.5"],
        test_requires=["pytest"],
    )



import subprocess
from unittest import mock

import pytest

from briefcase.console import Console, Log
from briefcase.platforms.macOS.app import macOSAppPackageCommand


@pytest.fixture
def package_command(tmp_path):
    command = macOSAppPackageCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )

    command.select_identity = mock.MagicMock()
    command.sign_app = mock.MagicMock()
    command.sign_file = mock.MagicMock()
    command.notarize = mock.MagicMock()
    command.dmgbuild = mock.MagicMock()
    command.tools.subprocess = mock.MagicMock(spec=subprocess)

    return command



import os
import sys
from collections import defaultdict
from unittest.mock import MagicMock

import pytest

from briefcase.console import Console, Log
from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.android_sdk import AndroidSDK
from briefcase.integrations.file import File
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.android.gradle import GradleOpenCommand

from ....utils import create_file


def create_sdk_manager(tmp_path, extension=""):
    create_file(
        tmp_path
        / "briefcase"
        / "tools"
        / "android_sdk"
        / "cmdline-tools"
        / AndroidSDK.SDK_MANAGER_VER
        / "bin"
        / f"sdkmanager{extension}",
        "Android SDK manager",
    )


@pytest.fixture
def open_command(tmp_path, first_app_config):
    command = GradleOpenCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.os = MagicMock(spec_set=os)
    command.tools.subprocess = MagicMock(spec_set=Subprocess)
    command.tools.file.download = MagicMock(spec_set=File.download)

    # Mock all apps as targeting version 0.3.15
    command._briefcase_toml = defaultdict(
        lambda: {"briefcase": {"target_version": "0.3.15"}}
    )

    # Mock some OS calls needed to make the tools appear to exist
    command.tools.os.environ = {}
    command.tools.os.access.return_value = True

    # Mock the file marking licenses as accepted
    create_file(
        tmp_path
        / "briefcase"
        / "tools"
        / "android_sdk"
        / "licenses"
        / "android-sdk-license",
        "license accepted",
    )

    return command


def test_unsupported_template_version(open_command, first_app_generated, tmp_path):
    """Error raised if template's target version is not supported."""
    # Skip tool verification
    open_command.verify_tools = MagicMock()

    open_command.verify_app = MagicMock(wraps=open_command.verify_app)

    open_command._briefcase_toml.update(
        {first_app_generated: {"briefcase": {"target_epoch": "0.3.16"}}}
    )

    with pytest.raises(
        BriefcaseCommandError,
        match="The app template used to generate this app is not compatible",
    ):
        open_command(first_app_generated)

    open_command.verify_app.assert_called_once_with(first_app_generated)


@pytest.mark.skipif(sys.platform != "darwin", reason="macOS specific test")
def test_open_macOS(open_command, first_app_config, tmp_path):
    """On macOS, open uses Finder to open the project folder."""
    # Mock the call to verify the existence of java
    open_command.tools.subprocess.check_output.return_value = "javac 17.0.7\n"

    # Create the project folder to mock a created project.
    open_command.project_path(first_app_config).mkdir(parents=True)

    # Create a stub sdkmanager
    create_sdk_manager(tmp_path)

    open_command(first_app_config)

    open_command.tools.subprocess.Popen.assert_called_once_with(
        [
            "open",
            tmp_path / "base_path/build/first-app/android/gradle",
        ]
    )


@pytest.mark.skipif(sys.platform != "linux", reason="Linux specific test")
def test_open_linux(open_command, first_app_config, tmp_path):
    """On linux, open invokes `xdg-open` on the project folder."""
    # Create the project folder to mock a created project.
    open_command.project_path(first_app_config).mkdir(parents=True)

    # Create a stub java binary
    create_file(tmp_path / "briefcase/tools/java17/bin/java", "java")

    # Create a stub sdkmanager
    create_sdk_manager(tmp_path)

    open_command(first_app_config)

    open_command.tools.subprocess.Popen.assert_called_once_with(
        [
            "xdg-open",
            tmp_path / "base_path/build/first-app/android/gradle",
        ]
    )


@pytest.mark.skipif(sys.platform != "win32", reason="Windows specific test")
def test_open_windows(open_command, first_app_config, tmp_path):
    """On Windows, open invokes `startfile` on the project folder."""
    # Create the project folder to mock a created project.
    open_command.project_path(first_app_config).mkdir(parents=True)

    # Create a stub java binary
    create_file(tmp_path / "briefcase/tools/java17/bin/java", "java")

    # Create a stub sdkmanager
    create_sdk_manager(tmp_path, extension=".bat")

    open_command(first_app_config)

    open_command.tools.os.startfile.assert_called_once_with(
        tmp_path / "base_path/build/first-app/android/gradle"
    )



import subprocess

import pytest

from briefcase.exceptions import BriefcaseCommandError


def test_upgrade(mock_tools, android_sdk):
    """Sdkmanager can be used to upgrade the Android SDK."""
    android_sdk.upgrade()

    mock_tools.subprocess.run.assert_called_once_with(
        [android_sdk.sdkmanager_path, "--update"],
        env=android_sdk.env,
        check=True,
        stream_output=False,
    )


def test_upgrade_failure(mock_tools, android_sdk):
    """If sdkmanager fails, an error is raised."""
    mock_tools.subprocess.run.side_effect = subprocess.CalledProcessError(1, "")
    with pytest.raises(BriefcaseCommandError):
        android_sdk.upgrade()

    mock_tools.subprocess.run.assert_called_once_with(
        [android_sdk.sdkmanager_path, "--update"],
        env=android_sdk.env,
        check=True,
        stream_output=False,
    )


def test_uninstall_does_nothing(mock_tools, android_sdk):
    """Uninstalling the Android SDK does nothing since it is upgraded in place."""
    android_sdk.uninstall()

    mock_tools.file.download.assert_not_called()
    mock_tools.shutil.unpack_archive.assert_not_called()
    mock_tools.subprocess.Popen.assert_not_called()
    mock_tools.subprocess.run.assert_not_called()
    mock_tools.subprocess.check_output.assert_not_called()



def test_missing_pyproject_file(convert_command):
    """An empty dict is returned if there is no pyproject file."""
    assert not (convert_command.base_path / "pyproject.toml").exists()
    assert convert_command.pyproject == {}


def test_present_pyproject_file(convert_command):
    """The pyproject.toml file is loaded if it is present."""
    (convert_command.base_path / "pyproject.toml").write_text(
        '[project]\nname="project-name"', encoding="utf-8"
    )
    assert convert_command.pyproject == {"project": {"name": "project-name"}}



from briefcase.bootstraps.base import BaseGuiBootstrap  # noqa: F401
from briefcase.bootstraps.console import ConsoleBootstrap  # noqa: F401
from briefcase.bootstraps.pygame import PygameGuiBootstrap  # noqa: F401
from briefcase.bootstraps.pyside6 import PySide6GuiBootstrap  # noqa: F401
from briefcase.bootstraps.toga import TogaGuiBootstrap  # noqa: F401



import pytest


@pytest.mark.parametrize(
    "url",
    [
        "https://example.com",
    ],
)
def test_valid_url(new_command, url):
    """Test that valid URLs are accepted."""
    assert new_command.validate_url(url)


@pytest.mark.parametrize(
    "url",
    [
        "not a URL!",  # Free text.
        "file:///usr/local/bin",  # File URL
        "gopher://example.com",  # URL, but not a webpage.
    ],
)
def test_invalid_url(new_command, url):
    """Test that invalid URLs are rejected."""
    with pytest.raises(ValueError):
        new_command.validate_url(url)



import os
import sys
from subprocess import CalledProcessError
from unittest.mock import MagicMock, PropertyMock

import pytest
import requests

from briefcase.console import Console, Log, LogLevel
from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.android_sdk import AndroidSDK
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.android.gradle import GradleBuildCommand


@pytest.fixture
def build_command(tmp_path, first_app_generated, monkeypatch):
    command = GradleBuildCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.android_sdk = MagicMock(spec_set=AndroidSDK)
    command.tools.os = MagicMock(spec_set=os)
    command.tools.os.environ = {}
    command.tools.sys = MagicMock(spec_set=sys)
    command.tools.requests = MagicMock(spec_set=requests)
    command.tools.subprocess = MagicMock(spec_set=Subprocess)
    monkeypatch.setattr(
        type(command.tools), "system_encoding", PropertyMock(return_value="ISO-42")
    )
    return command


def test_unsupported_template_version(build_command, first_app_generated):
    """Error raised if template's target version is not supported."""
    build_command.verify_app = MagicMock(wraps=build_command.verify_app)

    build_command._briefcase_toml.update(
        {first_app_generated: {"briefcase": {"target_epoch": "0.3.16"}}}
    )

    with pytest.raises(
        BriefcaseCommandError,
        match="The app template used to generate this app is not compatible",
    ):
        build_command(first_app_generated)

    build_command.verify_app.assert_called_once_with(first_app_generated)


@pytest.mark.parametrize(
    "host_os, gradlew_name, tool_debug_mode",
    [
        ("Windows", "gradlew.bat", True),
        ("Windows", "gradlew.bat", False),
        ("NonWindows", "gradlew", True),
        ("NonWindows", "gradlew", False),
    ],
)
def test_build_app(
    build_command,
    first_app_generated,
    host_os,
    gradlew_name,
    tool_debug_mode,
    tmp_path,
):
    """The app can be built, invoking gradle."""
    # Mock out `host_os` so we can validate which name is used for gradlew.
    build_command.tools.host_os = host_os
    # Enable verbose tool logging
    if tool_debug_mode:
        build_command.tools.logger.verbosity = LogLevel.DEEP_DEBUG
    # Create mock environment with `key`, which we expect to be preserved, and
    # `ANDROID_SDK_ROOT`, which we expect to be overwritten.
    build_command.tools.os.environ = {"ANDROID_SDK_ROOT": "somewhere", "key": "value"}
    build_command.build_app(first_app_generated, test_mode=False)
    build_command.tools.android_sdk.verify_emulator.assert_called_once_with()
    build_command.tools.subprocess.run.assert_called_once_with(
        [
            build_command.bundle_path(first_app_generated) / gradlew_name,
            "--console",
            "plain",
        ]
        + (["--debug"] if tool_debug_mode else [])
        + ["assembleDebug"],
        cwd=build_command.bundle_path(first_app_generated),
        env=build_command.tools.android_sdk.env,
        check=True,
        encoding="ISO-42",
    )

    # The app metadata contains the app module
    # The app metadata has been rewritten to reference the test module
    with (
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "android"
        / "gradle"
        / "res"
        / "briefcase.xml"
    ).open(encoding="utf-8") as f:
        assert (
            f.read()
            == "\n".join(
                [
                    "<resources>",
                    '    <string name="main_module">first_app</string>',
                    "</resources>",
                ]
            )
            + "\n"
        )


@pytest.mark.parametrize(
    "host_os, gradlew_name, debug_mode",
    [
        ("Windows", "gradlew.bat", True),
        ("Windows", "gradlew.bat", False),
        ("NonWindows", "gradlew", True),
        ("NonWindows", "gradlew", False),
    ],
)
def test_build_app_test_mode(
    build_command,
    first_app_generated,
    host_os,
    gradlew_name,
    debug_mode,
    tmp_path,
):
    """The app can be built in test mode, invoking gradle and rewriting app metadata."""
    # Mock out `host_os` so we can validate which name is used for gradlew.
    build_command.tools.host_os = host_os
    # Enable verbose tool logging
    if debug_mode:
        build_command.tools.logger.verbosity = LogLevel.DEEP_DEBUG
    # Create mock environment with `key`, which we expect to be preserved, and
    # `ANDROID_SDK_ROOT`, which we expect to be overwritten.
    build_command.tools.os.environ = {"ANDROID_SDK_ROOT": "somewhere", "key": "value"}
    build_command.build_app(first_app_generated, test_mode=True)
    build_command.tools.android_sdk.verify_emulator.assert_called_once_with()
    build_command.tools.subprocess.run.assert_called_once_with(
        [
            build_command.bundle_path(first_app_generated) / gradlew_name,
            "--console",
            "plain",
        ]
        + (["--debug"] if debug_mode else [])
        + ["assembleDebug"],
        cwd=build_command.bundle_path(first_app_generated),
        env=build_command.tools.android_sdk.env,
        check=True,
        encoding="ISO-42",
    )

    # The app metadata contains the app module
    # The app metadata has been rewritten to reference the test module
    with (
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "android"
        / "gradle"
        / "res"
        / "briefcase.xml"
    ).open(encoding="utf-8") as f:
        assert (
            f.read()
            == "\n".join(
                [
                    "<resources>",
                    '    <string name="main_module">tests.first_app</string>',
                    "</resources>",
                ]
            )
            + "\n"
        )


def test_print_gradle_errors(build_command, first_app_generated):
    """Validate that build_app() will convert stderr/stdout from the process into
    exception text."""
    # Create a mock subprocess that crashes, printing text partly in non-ASCII.
    build_command.tools.subprocess.run.side_effect = CalledProcessError(
        returncode=1,
        cmd=["ignored"],
    )
    with pytest.raises(BriefcaseCommandError):
        build_command.build_app(first_app_generated, test_mode=False)



import socket
import subprocess
import sys
from pathlib import Path, PosixPath, PurePosixPath
from unittest.mock import MagicMock, call

import pytest

from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.docker import XauthDatabaseCreationFailure

XAUTH_LIST_RET_1 = (
    "0000 0004 7f000101 0002 3132 0012 "
    "4d49542d4d414749432d434f4f4b49452d31 0010 1b185a208e4600e717f3b8903fc35141\n"
    "0000 0004 7f000101 0002 3130 0012 "
    "4d49542d4d414749432d434f4f4b49452d31 0010 da9667aa96def95dfa44e01f1b13e55b\n"
    "0100 0006 70692d372d35 0002 3133 0012 "
    "4d49542d4d414749432d434f4f4b49452d31 0010 72e93a771996c40f8465a7aa9add67e0\n"
    "0100 0006 70692d372d35 0002 3134 0012 "
    "4d49542d4d414749432d434f4f4b49452d31 0010 870c9e1d92ee663f93955d7fc27a3641\n"
    "0100 0006 70692d372d35 0002 3135 0012 "
    "4d49542d4d414749432d434f4f4b49452d31 0010 17d837a0890cdf5588bd793ef3410357\n"
    "0100 0006 70692d372d35 0002 3132 0012 "
    "4d49542d4d414749432d434f4f4b49452d31 0010 cb329aff1071b8bd66f24af31c8460cc\n"
    "0100 0006 70692d372d35 0002 3130 0012 "
    "4d49542d4d414749432d434f4f4b49452d31 0010 187dcffa78c868e5c5f67b6640c299f5\n"
    "0100 0006 70692d372d35 0002 3131 0012 "
    "4d49542d4d414749432d434f4f4b49452d31 0010 f22b85acf66a5819f5e15c2ac5af25b2"
)

XAUTH_LIST_RET_2 = (
    "0100 0007 6a757069746572 0000  0012 "
    "4d49542d4d414749432d434f4f4b49452d31 0010 fa4b61837675f1581427e0c937701439\n"
    "ffff 0007 6a757069746572 0000  0012 "
    "4d49542d4d414749432d434f4f4b49452d31 0010 fa4b61837675f1581427e0c937701439"
)


@pytest.mark.skipif(
    sys.platform == "win32", reason="Windows paths aren't converted in Docker context"
)
@pytest.mark.parametrize(
    "display_num, expected_socket_path",
    [("1", "/tmp/.X11-unix/X1"), (2, "/tmp/.X11-unix/X2")],
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_display_socket_path(mock_tools, display_num, expected_socket_path):
    """Path for socket is returned for X11 display."""
    socket_path = mock_tools.docker._x11_display_unix_socket_path(display_num)
    assert socket_path == PosixPath(expected_socket_path)


@pytest.mark.skipif(
    sys.platform == "win32", reason="Windows paths aren't converted in Docker context"
)
@pytest.mark.parametrize(
    "path_is_socket_outcome, expected_outcome",
    [(True, True), (OSError, False), (False, False)],
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_is_display_socket(mock_tools, path_is_socket_outcome, expected_outcome):
    """X11 display UNIX socket is validated for X display number."""
    # Mock the Path for the X display socket
    mock_socket_path = MagicMock(spec_set=PosixPath)
    mock_socket_path.is_socket.side_effect = [path_is_socket_outcome]
    mock_tools.docker._x11_display_unix_socket_path = MagicMock(
        return_value=mock_socket_path
    )

    assert mock_tools.docker._x11_is_display_unix(1) is expected_outcome


@pytest.mark.parametrize(
    "connect_ret, display_num, connect_port, expected_outcome",
    [
        (0, 0, 6000, True),
        (0, 100, 6100, True),
        (1, 42, 6042, False),
        (OSError, 1, 6001, False),
    ],
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_is_display_tcp(
    mock_tools,
    connect_ret,
    display_num,
    connect_port,
    expected_outcome,
    monkeypatch,
):
    """X11 display TCP socket is validated for X display number."""
    # Mock the socket manager
    mock_sock = MagicMock()
    mock_sock.connect_ex.side_effect = [connect_ret]
    mock_socket_manager = MagicMock()
    mock_socket_manager.__enter__.return_value = mock_sock
    mock_socket = MagicMock(return_value=mock_socket_manager)
    monkeypatch.setattr("socket.socket", mock_socket)

    assert mock_tools.docker._x11_is_display_tcp(display_num) is expected_outcome

    # Socket connection timeout is set
    mock_sock.settimeout.assert_called_once_with(3)
    # Connection type for socket is correct
    mock_socket.assert_called_once_with(socket.AF_INET, socket.SOCK_STREAM)
    # Connection attempt to the proper port
    mock_sock.connect_ex.assert_called_once_with(("localhost", connect_port))


@pytest.mark.parametrize(
    "is_socket_outcomes, is_tcp_outcomes, expected_display_num",
    [
        ([False], [False], 50),
        # Due to short-circuiting, only the first iterator is consumed if it returns False
        ([True, False], [False], 51),
        ([True, True, False, True, False], [False, False, False], 52),
        ([True] * 248 + [False, False], [True, False], 299),
    ],
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_allocate_display_success(
    mock_tools,
    is_socket_outcomes,
    is_tcp_outcomes,
    expected_display_num,
):
    """Allocating an X display number succeeds for a qualifying candidate."""
    # Mock X display socket checks
    mock_tools.docker._x11_is_display_unix = MagicMock(side_effect=is_socket_outcomes)
    mock_tools.docker._x11_is_display_tcp = MagicMock(side_effect=is_tcp_outcomes)

    assert mock_tools.docker._x11_allocate_display() == expected_display_num


@pytest.mark.parametrize(
    "is_socket_outcomes, is_tcp_outcomes",
    [
        ([True] * 250, [False] * 250),
        ([False] * 250, [True] * 250),
    ],
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_allocate_display_failure(mock_tools, is_socket_outcomes, is_tcp_outcomes):
    """Allocating an X display number fails if the range is exhausted."""
    # Mock X display socket checks
    mock_tools.docker._x11_is_display_unix = MagicMock(side_effect=is_socket_outcomes)
    mock_tools.docker._x11_is_display_tcp = MagicMock(side_effect=is_tcp_outcomes)

    with pytest.raises(BriefcaseCommandError, match="Failed to allocate X11 display"):
        mock_tools.docker._x11_allocate_display()


@pytest.mark.skipif(
    sys.platform == "win32", reason="Windows paths aren't converted in Docker context"
)
@pytest.mark.parametrize(
    "display_number, expected_xauth_path",
    [
        (1, Path.cwd() / "build/.briefcase.docker.xauth.1"),
        (50, Path.cwd() / "build/.briefcase.docker.xauth.50"),
    ],
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_proxy_display_xauth_file_path(
    mock_tools,
    display_number,
    expected_xauth_path,
):
    """The xauth database file path is generated correctly for the display number."""
    xauth_path = mock_tools.docker._x11_proxy_display_xauth_file_path(display_number)
    assert xauth_path == PosixPath(expected_xauth_path)


@pytest.mark.usefixtures("mock_docker")
def test_x11_write_xauth_missing_xauth_bin(mock_tools, tmp_path):
    """When the xauth program cannot be found, an error is raised."""
    mock_tools.shutil.which.return_value = ""

    with pytest.raises(BriefcaseCommandError, match="Install xauth to run an app"):
        mock_tools.docker._x11_write_xauth_file(
            DISPLAY=":66",
            xauth_file_path=tmp_path / "test_xauth_file",
            target_display_num=42,
        )


@pytest.mark.parametrize(
    "xauth_nlist_outcome",
    [
        subprocess.CalledProcessError(
            returncode=1, cmd=["xauth", "-i", "nlist", ":66"]
        ),
        "not found",
        "",
    ],
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_write_xauth_get_cookie_fails(mock_tools, xauth_nlist_outcome, tmp_path):
    """If the attempt to retrieve the current display's cookie fails, an error is
    raised."""

    mock_tools.subprocess._subprocess.check_output.side_effect = [xauth_nlist_outcome]

    with pytest.raises(
        XauthDatabaseCreationFailure, match="Failed to retrieve xauth cookie"
    ):
        mock_tools.docker._x11_write_xauth_file(
            DISPLAY=":66",
            xauth_file_path=tmp_path / "test_xauth_file",
            target_display_num=42,
        )


@pytest.mark.usefixtures("mock_docker")
def test_x11_write_xauth_add_new_xauth_fails(mock_tools, tmp_path):
    """If the attempt to retrieve the current display's cookie fails, an error is
    raised."""

    mock_tools.subprocess._subprocess.check_output.side_effect = [
        # xauth -i nlist :66
        XAUTH_LIST_RET_1,
        subprocess.CalledProcessError(
            returncode=1,
            cmd=[
                "xauth",
                "-i",
                "-f",
                str(tmp_path / "test_xauth_file"),
                "add",
                ":66",
                "MIT-MAGIC-COOKIE-1",
                "cookie",
            ],
        ),
    ]

    with pytest.raises(
        XauthDatabaseCreationFailure,
        match="Failed to add a xauth entry for existing cookie",
    ):
        mock_tools.docker._x11_write_xauth_file(
            DISPLAY=":66",
            xauth_file_path=tmp_path / "test_xauth_file",
            target_display_num=42,
        )


@pytest.mark.usefixtures("mock_docker")
def test_x11_write_xauth_retrieve_xauth_fails(mock_tools, tmp_path):
    """If the attempt to retrieve the target display's xauth list, an error is
    raised."""

    mock_tools.subprocess._subprocess.check_output.side_effect = [
        # xauth -i nlist :66
        XAUTH_LIST_RET_1,
        # xauth -i -f "xauth_file_path" add "MIT-MAGIC-COOKIE-1" <cookie>
        "xauth add success",
        subprocess.CalledProcessError(
            returncode=1,
            cmd=["xauth", "-i", "-f", str(tmp_path / "test_xauth_file"), "nlist"],
        ),
    ]

    with pytest.raises(
        XauthDatabaseCreationFailure, match="Failed to retrieve xauth list"
    ):
        mock_tools.docker._x11_write_xauth_file(
            DISPLAY=":66",
            xauth_file_path=tmp_path / "test_xauth_file",
            target_display_num=42,
        )


@pytest.mark.usefixtures("mock_docker")
def test_x11_write_xauth_merge_xauth_fails(mock_tools, tmp_path):
    """If the attempt to retrieve the target display's xauth list, an error is
    raised."""

    mock_tools.subprocess._subprocess.check_output.side_effect = [
        # xauth -i nlist :66
        XAUTH_LIST_RET_1,
        # xauth -i -f "xauth_file_path" add "MIT-MAGIC-COOKIE-1" <cookie>
        "xauth add success",
        # xauth -i -f "xauth_file_path" nlist
        XAUTH_LIST_RET_2,
        subprocess.CalledProcessError(
            returncode=1,
            cmd=["xauth", "-f", str(tmp_path / "test_xauth_file"), "nmerge", "-"],
        ),
    ]

    with pytest.raises(
        XauthDatabaseCreationFailure,
        match="Failed to merge xauth updates for FamilyWild hostname",
    ):
        mock_tools.docker._x11_write_xauth_file(
            DISPLAY=":66",
            xauth_file_path=tmp_path / "test_xauth_file",
            target_display_num=42,
        )


@pytest.mark.skipif(
    sys.platform == "win32", reason="Windows paths aren't converted in Docker context"
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_write_xauth_success(mock_tools, tmp_path, sub_check_output_kw):
    """The attempt to write the xauth database for the target display succeeds."""

    mock_tools.subprocess._subprocess.check_output.side_effect = [
        # xauth -i nlist :66
        XAUTH_LIST_RET_1,
        # xauth -i -f "xauth_file_path" add "MIT-MAGIC-COOKIE-1" <cookie>
        "xauth add success",
        # xauth -i -f "xauth_file_path" nlist
        XAUTH_LIST_RET_2,
        # xauth -i -f "xauth_file_path" nmerge -
        "xauth nmerge success",
    ]

    mock_tools.docker._x11_write_xauth_file(
        DISPLAY=":66",
        xauth_file_path=tmp_path / "test_xauth_file",
        target_display_num=42,
    )

    mock_tools.subprocess._subprocess.check_output.assert_has_calls(
        [
            call(["xauth", "-i", "nlist", ":66"], **sub_check_output_kw),
            call(
                [
                    "xauth",
                    "-i",
                    "-f",
                    f"{tmp_path}/test_xauth_file",
                    "add",
                    ":42",
                    "MIT-MAGIC-COOKIE-1",
                    "1b185a208e4600e717f3b8903fc35141",
                ],
                **sub_check_output_kw,
            ),
            call(
                [
                    "xauth",
                    "-i",
                    "-f",
                    f"{tmp_path}/test_xauth_file",
                    "nlist",
                ],
                **sub_check_output_kw,
            ),
            call(
                [
                    "xauth",
                    "-i",
                    "-f",
                    f"{tmp_path}/test_xauth_file",
                    "nmerge",
                    "-",
                ],
                input=(
                    "ffff 0007 6a757069746572 0000  0012 4d49542d4d414749432d434f4f4b49452d31 "
                    "0010 fa4b61837675f1581427e0c937701439\n"
                    "ffff 0007 6a757069746572 0000  0012 4d49542d4d414749432d434f4f4b49452d31 "
                    "0010 fa4b61837675f1581427e0c937701439"
                ),
                **sub_check_output_kw,
            ),
        ],
    )


@pytest.mark.usefixtures("mock_docker")
def test_x11_tcp_proxy_missing_socat_bin(mock_tools):
    """When the socat program cannot be found, an error is raised."""
    mock_tools.shutil.which.return_value = ""

    with pytest.raises(
        BriefcaseCommandError,
        match="Install socat to run an app for a targeted Linux distribution",
    ):
        mock_tools.docker._x11_tcp_proxy(DISPLAY=":66")


@pytest.mark.parametrize("DISPLAY", [None, "", "host;42.0", 42, "42"])
@pytest.mark.usefixtures("mock_docker")
def test_x11_tcp_proxy_invalid_DISPLAY(mock_tools, DISPLAY):
    """When the DISPLAY environment variable is invalid, an error is raised."""
    with pytest.raises(
        BriefcaseCommandError,
        match=f"Unsupported value for environment variable DISPLAY: '{DISPLAY}'",
    ):
        mock_tools.docker._x11_tcp_proxy(DISPLAY=DISPLAY)


@pytest.mark.usefixtures("mock_docker")
def test_x11_tcp_proxy_unknown_display_socket(mock_tools):
    """When a socket for the current display cannot be found, an error is raised."""
    # Mock target display allocation
    mock_tools.docker._x11_allocate_display = MagicMock(return_value=42)
    # Mock current display sockets cannot be found
    mock_tools.docker._x11_is_display_tcp = MagicMock(return_value=False)
    mock_tools.docker._x11_is_display_unix = MagicMock(return_value=False)

    with pytest.raises(
        BriefcaseCommandError,
        match="Cannot find X11 display for ':42'",
    ):
        mock_tools.docker._x11_tcp_proxy(DISPLAY=":42")


@pytest.mark.parametrize(
    "is_unix, is_tcp, current_display_num, target_display_num, connect_def",
    [
        (False, True, 142, 42, "TCP:localhost:{tcp_port}"),
        (False, True, 166, 66, "TCP:localhost:{tcp_port}"),
        (True, True, 166, 66, "TCP:localhost:{tcp_port}"),
        (True, False, 142, 42, "UNIX-CONNECT:/tmp/.X11-unix/X{display_num}"),
        (True, False, 166, 66, "UNIX-CONNECT:/tmp/.X11-unix/X{display_num}"),
    ],
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_tcp_proxy_create_success(
    mock_tools,
    is_unix,
    is_tcp,
    current_display_num,
    target_display_num,
    connect_def,
    sub_kw,
):
    """When current display has a socket, the proxy is to that TCP socket."""
    # Mock target display allocation
    mock_tools.docker._x11_allocate_display = MagicMock(return_value=target_display_num)
    # Mock current display sockets cannot be found
    mock_tools.docker._x11_is_display_tcp = MagicMock(return_value=is_tcp)
    mock_tools.docker._x11_is_display_unix = MagicMock(return_value=is_unix)
    # Mock proxy process
    mock_proxy_popen = MagicMock(spec_set=subprocess.Popen)
    mock_tools.subprocess._subprocess.Popen.return_value = mock_proxy_popen

    proxy_popen, proxy_display_num = mock_tools.docker._x11_tcp_proxy(
        DISPLAY=f"host:{current_display_num}.0"
    )

    # Proxy and spoofed target display are returned
    assert proxy_popen is mock_proxy_popen
    assert proxy_display_num == target_display_num

    # Proxy created properly
    mock_tools.subprocess._subprocess.Popen.assert_called_with(
        [
            "socat",
            f"TCP-LISTEN:{6000 + target_display_num},reuseaddr,fork,bind=0.0.0.0",
            connect_def.format(
                tcp_port=6000 + current_display_num,
                display_num=current_display_num,
            ),
        ],
        **sub_kw,
    )


@pytest.mark.parametrize("DISPLAY", [None, ""])
@pytest.mark.usefixtures("mock_docker")
def test_x11_passthrough_missing_DISPLAY(mock_tools, DISPLAY):
    """If the DISPLAY variable is not set, an error is raised."""
    # Mock DISPLAY environment variable
    mock_tools.os.getenv.return_value = DISPLAY

    with pytest.raises(
        BriefcaseCommandError,
        match="The DISPLAY environment variable must be set to run an app in Docker",
    ):
        with mock_tools.docker.x11_passthrough({}):
            pass


@pytest.mark.skipif(
    sys.platform == "win32", reason="Windows paths aren't converted in Docker context"
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_passthrough_fails(mock_tools):
    """Cleanup runs even if passthrough fails."""
    # Mock DISPLAY environment variable
    mock_tools.os.getenv.return_value = "host:42.0"
    # Mock Subprocess.cleanup()
    mock_tools.subprocess.cleanup = MagicMock()
    # Mock the proxy
    mock_proxy_popen = MagicMock(spec_set=subprocess.Popen)
    mock_tools.docker._x11_tcp_proxy = MagicMock(return_value=(mock_proxy_popen, 66))
    # Mock xauth database file path
    mock_xauth_file_path = MagicMock(spec_set=PosixPath)
    mock_tools.docker._x11_proxy_display_xauth_file_path = MagicMock(
        return_value=mock_xauth_file_path
    )
    # Mock the xauth database file write failing (beyond what's expected)
    mock_tools.docker._x11_write_xauth_file = MagicMock(
        side_effect=OSError("write failed")
    )

    with pytest.raises(OSError, match="write failed"):
        with mock_tools.docker.x11_passthrough({}):
            pass

    # Proxy cleanup ran
    mock_tools.subprocess.cleanup.assert_called_once_with(
        "X display proxy", mock_proxy_popen
    )
    mock_xauth_file_path.unlink.assert_called_once_with(missing_ok=True)


@pytest.mark.skipif(
    sys.platform == "win32", reason="Windows paths aren't converted in Docker context"
)
@pytest.mark.parametrize(
    "in_kwargs, out_kwargs",
    [
        (
            {
                "input1": "value1",
                "input2": "value2",
                "env": {"existing-var": "existing-val"},
                "mounts": [("/outside", "/inside")],
                "add_hosts": [("source", "target")],
            },
            {
                "input1": "value1",
                "input2": "value2",
                "env": {
                    "existing-var": "existing-val",
                    "DISPLAY": "host.docker.internal:66",
                },
                "mounts": [
                    ("/outside", "/inside"),
                ],
                "add_hosts": [
                    ("source", "target"),
                    ("host.docker.internal", "host-gateway"),
                ],
            },
        ),
        (
            {},
            {
                "env": {
                    "DISPLAY": "host.docker.internal:66",
                },
                "add_hosts": [("host.docker.internal", "host-gateway")],
            },
        ),
    ],
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_passthrough_xauth_fails(mock_tools, in_kwargs, out_kwargs, capsys):
    """Passthrough configuration is successful even if xauth fails and cleanup runs."""
    # Mock DISPLAY environment variable
    mock_tools.os.getenv.return_value = "host:42.0"
    # Mock Subprocess.cleanup()
    mock_tools.subprocess.cleanup = MagicMock()
    # Mock the proxy
    mock_proxy_popen = MagicMock(spec_set=subprocess.Popen)
    mock_tools.docker._x11_tcp_proxy = MagicMock(return_value=(mock_proxy_popen, 66))
    # Mock xauth database file path
    mock_xauth_file_path = MagicMock(spec_set=PosixPath)
    mock_tools.docker._x11_proxy_display_xauth_file_path = MagicMock(
        return_value=mock_xauth_file_path
    )
    # Mock the xauth database file write failing
    mock_tools.docker._x11_write_xauth_file = MagicMock(
        side_effect=XauthDatabaseCreationFailure("write failed")
    )

    with mock_tools.docker.x11_passthrough(in_kwargs) as kwargs:
        assert kwargs == out_kwargs

    # Proxy cleanup ran
    mock_tools.subprocess.cleanup.assert_called_once_with(
        "X display proxy", mock_proxy_popen
    )
    mock_xauth_file_path.unlink.assert_called_once_with(missing_ok=True)

    # User is warned that xauth failed
    assert capsys.readouterr().out == (
        "An X11 authentication database could not be created for the display.\n"
        "\n"
        "Briefcase will proceed, but if access to the display is rejected, this may be why.\n"
    )


@pytest.mark.skipif(
    sys.platform == "win32", reason="Windows paths aren't converted in Docker context"
)
@pytest.mark.parametrize(
    "in_kwargs, out_kwargs",
    [
        (
            {
                "input1": "value1",
                "input2": "value2",
                "env": {"existing-var": "existing-val"},
                "mounts": [("/outside", "/inside")],
                "add_hosts": [("source", "target")],
            },
            {
                "input1": "value1",
                "input2": "value2",
                "env": {
                    "existing-var": "existing-val",
                    "DISPLAY": "host.docker.internal:66",
                    "XAUTHORITY": "<placeholder>",
                },
                "mounts": [
                    ("/outside", "/inside"),
                ],
                "add_hosts": [
                    ("source", "target"),
                    ("host.docker.internal", "host-gateway"),
                ],
            },
        ),
        (
            {},
            {
                "env": {
                    "DISPLAY": "host.docker.internal:66",
                    "XAUTHORITY": "<placeholder>",
                },
                "mounts": [],
                "add_hosts": [("host.docker.internal", "host-gateway")],
            },
        ),
    ],
)
@pytest.mark.usefixtures("mock_docker")
def test_x11_passthrough_success(mock_tools, in_kwargs, out_kwargs, capsys):
    """Passthrough configuration is successful and cleanup runs."""
    # Mock DISPLAY environment variable
    mock_tools.os.getenv.return_value = "host:42.0"
    # Mock Subprocess.cleanup()
    mock_tools.subprocess.cleanup = MagicMock()
    # Mock the proxy
    mock_proxy_popen = MagicMock(spec_set=subprocess.Popen)
    mock_tools.docker._x11_tcp_proxy = MagicMock(return_value=(mock_proxy_popen, 66))
    # Mock xauth database file path
    mock_xauth_file_path = MagicMock(wraps=PosixPath("/tmp/subdir/xauth_file.db"))
    mock_xauth_file_path.name = "xauth_file.db"
    mock_tools.docker._x11_proxy_display_xauth_file_path = MagicMock(
        return_value=mock_xauth_file_path
    )
    # Mock the xauth database file write
    mock_tools.docker._x11_write_xauth_file = MagicMock()

    # Finish filling out augmented subprocess kwargs
    out_kwargs.setdefault("mounts", []).append(
        (mock_xauth_file_path, PurePosixPath("/tmp/xauth_file.db"))
    )
    out_kwargs.setdefault("env", {})["XAUTHORITY"] = PurePosixPath("/tmp/xauth_file.db")

    with mock_tools.docker.x11_passthrough(subprocess_kwargs=in_kwargs) as kwargs:
        assert kwargs == out_kwargs

    # Proxy cleanup ran
    mock_tools.subprocess.cleanup.assert_called_once_with(
        "X display proxy", mock_proxy_popen
    )
    mock_xauth_file_path.unlink.assert_called_once_with(missing_ok=True)

    # No console output
    assert capsys.readouterr().out == ""



import os
from unittest import mock

import pytest

import briefcase.integrations.xcode
from briefcase.exceptions import BriefcaseCommandError


def test_package_formats(package_command):
    """Packaging formats are as expected."""
    assert package_command.packaging_formats == ["zip", "dmg", "pkg"]
    # The default format is encoded as None, and then updated
    # as part of app verification.
    assert package_command.default_packaging_format is None


@pytest.mark.parametrize(
    "is_console_app, packaging_format, actual_format",
    [
        (False, None, "dmg"),  # default for GUI app is DMG
        (False, "dmg", "dmg"),
        (False, "app", "app"),
        (False, "pkg", "pkg"),
        (True, None, "pkg"),  # default for console app is PKG
        (True, "pkg", "pkg"),
    ],
)
def test_effective_format(
    package_command,
    first_app_with_binaries,
    is_console_app,
    packaging_format,
    actual_format,
):
    """The packaging format varies depending on the app type."""

    first_app_with_binaries.packaging_format = packaging_format
    first_app_with_binaries.console_app = is_console_app
    package_command.verify_app(first_app_with_binaries)

    assert first_app_with_binaries.packaging_format == actual_format


@pytest.mark.parametrize("packaging_format", ["zip", "dmg"])
def test_console_invalid_formats(
    package_command,
    first_app_with_binaries,
    packaging_format,
):
    """Some packaging formats are not valid for console apps."""

    first_app_with_binaries.packaging_format = packaging_format
    first_app_with_binaries.console_app = True
    with pytest.raises(
        BriefcaseCommandError,
        match=r"macOS console apps must be distributed in PKG format\.",
    ):
        package_command.verify_app(first_app_with_binaries)


def test_no_notarize_option(package_command):
    """The --no-notarize option can be parsed."""
    options, overrides = package_command.parse_options(["--no-notarize"])

    assert options == {
        "adhoc_sign": False,
        "identity": None,
        "notarize_app": False,
        "installer_identity": None,
        "sign_installer": True,
        "packaging_format": None,
        "update": False,
    }
    assert overrides == {}


def test_installer_identity_option(package_command):
    """The --installer_identity option can be parsed."""
    options, overrides = package_command.parse_options(
        ["--installer-identity", "DEADBEEF"]
    )

    assert options == {
        "adhoc_sign": False,
        "identity": None,
        "notarize_app": None,
        "installer_identity": "DEADBEEF",
        "sign_installer": True,
        "packaging_format": None,
        "update": False,
    }
    assert overrides == {}


def test_no_sign_installer(package_command):
    """The --no-sign-installer option can be parsed."""
    options, overrides = package_command.parse_options(["--no-sign-installer"])

    assert options == {
        "adhoc_sign": False,
        "identity": None,
        "notarize_app": None,
        "installer_identity": None,
        "sign_installer": False,
        "packaging_format": None,
        "update": False,
    }
    assert overrides == {}


def test_verify(package_command, monkeypatch):
    """If you're on macOS, you can verify tools."""
    package_command.tools.host_os = "Darwin"

    # Mock the existence of the command line tools
    mock_ensure_command_line_tools_are_installed = mock.MagicMock()
    monkeypatch.setattr(
        briefcase.integrations.xcode.XcodeCliTools,
        "ensure_command_line_tools_are_installed",
        mock_ensure_command_line_tools_are_installed,
    )
    mock_confirm_xcode_license_accepted = mock.MagicMock()
    monkeypatch.setattr(
        briefcase.integrations.xcode.XcodeCliTools,
        "confirm_xcode_license_accepted",
        mock_confirm_xcode_license_accepted,
    )

    package_command.verify_tools()

    assert package_command.tools.xcode_cli is not None
    mock_ensure_command_line_tools_are_installed.assert_called_once_with(
        tools=package_command.tools
    )
    mock_confirm_xcode_license_accepted.assert_called_once_with(
        tools=package_command.tools
    )


def test_package_app(
    package_command,
    first_app_with_binaries,
    sekrit_identity,
    tmp_path,
    capsys,
):
    """A macOS App is packaged as a signed, notarized DMG by default."""
    # Select a codesigning identity
    package_command.select_identity.return_value = sekrit_identity

    # Package the app. Sign and notarize by default
    package_command.package_app(first_app_with_binaries)

    # A request has been made to sign the app
    package_command.sign_app.assert_called_once_with(
        app=first_app_with_binaries,
        identity=sekrit_identity,
    )

    # The DMG has been built as expected
    package_command.dmgbuild.build_dmg.assert_called_once_with(
        filename=os.fsdecode(tmp_path / "base_path/dist/First App-0.0.1.dmg"),
        volume_name="First App 0.0.1",
        settings={
            "files": [
                os.fsdecode(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "macos"
                    / "app"
                    / "First App.app"
                )
            ],
            "symlinks": {"Applications": "/Applications"},
            "icon_locations": {
                "First App.app": (75, 75),
                "Applications": (225, 75),
            },
            "window_rect": ((600, 600), (350, 150)),
            "icon_size": 64,
            "text_size": 12,
        },
    )

    # A request was made to sign the DMG as well.
    # This ignores the calls that would have been made transitively
    # by calling sign_app()
    package_command.sign_file.assert_called_once_with(
        tmp_path / "base_path/dist/First App-0.0.1.dmg",
        identity=sekrit_identity,
    )

    # A request was made to notarize the DMG
    package_command.notarize.assert_called_once_with(
        tmp_path / "base_path/dist/First App-0.0.1.dmg",
        identity=sekrit_identity,
    )

    # The app doesn't specify an app icon or installer icon, so there's no
    # mention about the DMG installer icon in the console log.
    assert "DMG installer icon" not in capsys.readouterr().out


def test_no_notarization(
    package_command,
    first_app_with_binaries,
    sekrit_identity,
    tmp_path,
    capsys,
):
    """A macOS App can be packaged as a signed DMG without notarization."""
    # Select a codesigning identity
    package_command.select_identity.return_value = sekrit_identity

    # Package the app; sign by default, but disable notarization
    package_command.package_app(first_app_with_binaries, notarize_app=False)

    # A request has been made to sign the app
    package_command.sign_app.assert_called_once_with(
        app=first_app_with_binaries,
        identity=sekrit_identity,
    )

    # The DMG has been built as expected
    package_command.dmgbuild.build_dmg.assert_called_once_with(
        filename=os.fsdecode(tmp_path / "base_path/dist/First App-0.0.1.dmg"),
        volume_name="First App 0.0.1",
        settings={
            "files": [
                os.fsdecode(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "macos"
                    / "app"
                    / "First App.app"
                )
            ],
            "symlinks": {"Applications": "/Applications"},
            "icon_locations": {
                "First App.app": (75, 75),
                "Applications": (225, 75),
            },
            "window_rect": ((600, 600), (350, 150)),
            "icon_size": 64,
            "text_size": 12,
        },
    )

    # A request was made to sign the DMG as well.
    # This ignores the calls that would have been made transitively
    # by calling sign_app()
    package_command.sign_file.assert_called_once_with(
        tmp_path / "base_path/dist/First App-0.0.1.dmg",
        identity=sekrit_identity,
    )

    # A request was made to notarize the DMG
    package_command.notarize.assert_not_called()

    # The app doesn't specify an app icon or installer icon, so there's no
    # mention about the DMG installer icon in the console log.
    assert "DMG installer icon" not in capsys.readouterr().out


def test_adhoc_sign(
    package_command,
    first_app_with_binaries,
    adhoc_identity,
    tmp_path,
):
    """A macOS App can be packaged and signed with ad-hoc identity."""
    # Package the app with an ad-hoc identity.
    # Explicitly disable notarization (can't ad-hoc notarize an app)
    package_command.package_app(
        first_app_with_binaries,
        adhoc_sign=True,
        notarize_app=False,
    )

    # A request has been made to sign the app
    package_command.sign_app.assert_called_once_with(
        app=first_app_with_binaries,
        identity=adhoc_identity,
    )

    # The DMG has been built as expected
    package_command.dmgbuild.build_dmg.assert_called_once_with(
        filename=os.fsdecode(tmp_path / "base_path/dist/First App-0.0.1.dmg"),
        volume_name="First App 0.0.1",
        settings={
            "files": [
                os.fsdecode(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "macos"
                    / "app"
                    / "First App.app"
                )
            ],
            "symlinks": {"Applications": "/Applications"},
            "icon_locations": {
                "First App.app": (75, 75),
                "Applications": (225, 75),
            },
            "window_rect": ((600, 600), (350, 150)),
            "icon_size": 64,
            "text_size": 12,
        },
    )

    # A request was made to sign the DMG as well.
    # This ignores the calls that would have been made transitively
    # by calling sign_app()
    package_command.sign_file.assert_called_once_with(
        tmp_path / "base_path/dist/First App-0.0.1.dmg",
        identity=adhoc_identity,
    )

    # No request was made to notarize
    package_command.notarize.assert_not_called()


def test_notarize_adhoc_signed(package_command, first_app_with_binaries):
    """A macOS App cannot be notarized if ad-hoc signing is requested."""

    # Package the app without code signing
    with pytest.raises(
        BriefcaseCommandError,
        match=r"Can't notarize an app with an ad-hoc signing identity",
    ):
        package_command.package_app(
            first_app_with_binaries,
            notarize_app=True,
            adhoc_sign=True,
        )

    # No code signing or notarization has been performed.
    assert package_command.select_identity.call_count == 0
    assert package_command.sign_app.call_count == 0
    assert package_command.sign_file.call_count == 0
    assert package_command.notarize.call_count == 0


def test_notarize_adhoc_signed_via_prompt(
    package_command,
    first_app_with_binaries,
    adhoc_identity,
):
    """Notarization is rejected if the user selects the adhoc identity."""

    package_command.select_identity.return_value = adhoc_identity

    # Package the app without code signing
    with pytest.raises(
        BriefcaseCommandError,
        match=r"Can't notarize an app with an ad-hoc signing identity",
    ):
        package_command.package_app(
            first_app_with_binaries,
            notarize_app=True,
        )

    # No code signing or notarization has been performed.
    assert package_command.select_identity.call_count == 1
    assert package_command.sign_app.call_count == 0
    assert package_command.sign_file.call_count == 0
    assert package_command.notarize.call_count == 0


def test_adhoc_sign_default_no_notarization(
    package_command,
    first_app_with_binaries,
    adhoc_identity,
    tmp_path,
):
    """An ad-hoc signed app is not notarized by default."""
    # Package the app with an ad-hoc identity; notarization will
    # be disabled as a default
    package_command.package_app(
        first_app_with_binaries,
        adhoc_sign=True,
    )

    # A request has been made to sign the app
    package_command.sign_app.assert_called_once_with(
        app=first_app_with_binaries,
        identity=adhoc_identity,
    )

    # The DMG has been built as expected
    package_command.dmgbuild.build_dmg.assert_called_once_with(
        filename=os.fsdecode(tmp_path / "base_path/dist/First App-0.0.1.dmg"),
        volume_name="First App 0.0.1",
        settings={
            "files": [
                os.fsdecode(
                    tmp_path
                    / "base_path"
                    / "build"
                    / "first-app"
                    / "macos"
                    / "app"
                    / "First App.app"
                )
            ],
            "symlinks": {"Applications": "/Applications"},
            "icon_locations": {
                "First App.app": (75, 75),
                "Applications": (225, 75),
            },
            "window_rect": ((600, 600), (350, 150)),
            "icon_size": 64,
            "text_size": 12,
        },
    )

    # A request was made to sign the DMG as well.
    # This ignores the calls that would have been made transitively
    # by calling sign_app()
    package_command.sign_file.assert_called_once_with(
        tmp_path / "base_path/dist/First App-0.0.1.dmg",
        identity=adhoc_identity,
    )

    # No request was made to notarize
    package_command.notarize.assert_not_called()


def test_sign_failure(
    package_command,
    first_app_with_binaries,
    sekrit_identity,
    tmp_path,
):
    """If the signing process can't be completed, an error is raised."""

    # Select a codesigning identity
    package_command.select_identity.return_value = sekrit_identity

    # Raise an error when attempting to sign the app
    package_command.sign_app.side_effect = BriefcaseCommandError("Unable to code sign")

    # Attempt to package the app; it should raise an error
    with pytest.raises(BriefcaseCommandError, match=r"Unable to code sign"):
        package_command.package_app(first_app_with_binaries)

    # A request has been made to sign the app
    package_command.sign_app.assert_called_once_with(
        app=first_app_with_binaries,
        identity=sekrit_identity,
    )

    # dmgbuild has not been called
    package_command.dmgbuild.build_dmg.assert_not_called()

    # No attempt was made to sign the dmg either
    # This ignores the calls that would have been made transitively
    # by calling sign_app()
    package_command.sign_file.assert_not_called()



import os
from unittest.mock import MagicMock

import pytest

from briefcase.exceptions import (
    BriefcaseCommandError,
    MissingToolError,
    NetworkFailure,
    NonManagedToolError,
)
from briefcase.integrations.wix import WiX

from ...utils import create_zip_file
from .conftest import WIX_DOWNLOAD_URL


def test_non_managed_install(mock_tools, tmp_path, capsys):
    """If the WiX install points to a non-managed install, no upgrade is attempted."""

    # Make the installation point to somewhere else.
    wix = WiX(mock_tools, wix_home=tmp_path / "other-WiX")

    # Attempt an upgrade. This will fail because the install is non-managed
    with pytest.raises(NonManagedToolError):
        wix.upgrade()

    # No download was attempted
    assert mock_tools.file.download.call_count == 0


def test_non_existing_wix_install(mock_tools, tmp_path):
    """If there's no existing managed WiX install, upgrading is an error."""
    # Create an SDK wrapper around a non-existing managed install
    wix = WiX(mock_tools, wix_home=tmp_path / "tools/wix")

    with pytest.raises(MissingToolError):
        wix.upgrade()

    # No download was attempted
    assert mock_tools.file.download.call_count == 0


def test_existing_wix_install(mock_tools, tmp_path):
    """If there's an existing managed WiX install, it is deleted and redownloaded."""
    # Create a mock of a previously installed WiX version.
    wix_path = tmp_path / "tools/wix"
    wix_path.mkdir(parents=True)
    (wix_path / "heat.exe").touch()
    (wix_path / "light.exe").touch()
    (wix_path / "candle.exe").touch()

    # Mock the download
    wix_path = tmp_path / "tools/wix"

    wix_zip_path = create_zip_file(tmp_path / "tools/wix.zip", content=[("wix", "wix")])

    mock_tools.file.download = MagicMock(return_value=wix_zip_path)

    # Create an SDK wrapper
    wix = WiX(mock_tools, wix_home=wix_path, bin_install=True)

    # Attempt an upgrade.
    wix.upgrade()

    # The old version has been deleted
    mock_tools.shutil.rmtree.assert_called_with(wix_path)

    # A download was initiated
    mock_tools.file.download.assert_called_with(
        url=WIX_DOWNLOAD_URL,
        download_path=tmp_path / "tools",
        role="WiX",
    )

    # The download was unpacked
    mock_tools.shutil.unpack_archive.assert_called_with(
        filename=os.fsdecode(wix_zip_path), extract_dir=os.fsdecode(wix_path)
    )

    # The zip file was removed
    assert not wix_zip_path.exists()


def test_download_fail(mock_tools, tmp_path):
    """If the download doesn't complete, the upgrade fails."""
    # Create a mock of a previously installed WiX version.
    wix_path = tmp_path / "tools/wix"
    wix_path.mkdir(parents=True)
    (wix_path / "heat.exe").touch()
    (wix_path / "light.exe").touch()
    (wix_path / "candle.exe").touch()

    # Mock the download failure
    mock_tools.file.download = MagicMock(side_effect=NetworkFailure("mock"))

    # Create an SDK wrapper
    wix = WiX(mock_tools, wix_home=wix_path, bin_install=True)

    # Upgrade the install. This will trigger a download that will fail
    with pytest.raises(NetworkFailure, match="Unable to mock"):
        wix.upgrade()

    # A download was initiated
    mock_tools.file.download.assert_called_with(
        url=WIX_DOWNLOAD_URL,
        download_path=tmp_path / "tools",
        role="WiX",
    )

    # ... but the unpack didn't happen
    assert mock_tools.shutil.unpack_archive.call_count == 0


def test_unpack_fail(mock_tools, tmp_path):
    """If the download archive is corrupted, the validator fails."""
    # Create a mock of a previously installed WiX version.
    wix_path = tmp_path / "tools/wix"
    wix_path.mkdir(parents=True)
    (wix_path / "heat.exe").touch()
    (wix_path / "light.exe").touch()
    (wix_path / "candle.exe").touch()

    # Mock the download
    wix_zip_path = create_zip_file(tmp_path / "tools/wix.zip", content=[("wix", "wix")])

    mock_tools.file.download = MagicMock(return_value=wix_zip_path)

    # Mock an unpack failure
    mock_tools.shutil.unpack_archive.side_effect = EOFError

    # Create an SDK wrapper
    wix = WiX(mock_tools, wix_home=wix_path, bin_install=True)

    # Upgrade the install. This will trigger a download,
    # but the unpack will fail.
    with pytest.raises(BriefcaseCommandError):
        wix.upgrade()

    # A download was initiated
    mock_tools.file.download.assert_called_with(
        url=WIX_DOWNLOAD_URL,
        download_path=tmp_path / "tools",
        role="WiX",
    )

    # The download was unpacked.
    mock_tools.shutil.unpack_archive.assert_called_with(
        filename=os.fsdecode(wix_zip_path), extract_dir=os.fsdecode(wix_path)
    )

    # The zip file was not removed
    assert wix_zip_path.exists()



import pytest


# Windows' AppConfig requires attribute 'packaging_format'
@pytest.fixture
def first_app_config(first_app_config):
    first_app_config.packaging_format = "msi"
    return first_app_config



import subprocess
from unittest.mock import MagicMock, call

import pytest

from briefcase.console import Console, Log
from briefcase.exceptions import BriefcaseCommandError
from briefcase.platforms.linux.system import LinuxSystemBuildCommand


@pytest.fixture
def build_command(tmp_path, first_app):
    command = LinuxSystemBuildCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
        apps={"first": first_app},
    )
    command.tools.host_os = "Linux"
    command.tools.host_arch = "wonky"

    # All calls to `shutil.which()` succeed
    command.tools.shutil.which = MagicMock(return_value="/path/to/exe")

    # Mock subprocess
    command.tools.subprocess = MagicMock()

    return command


def test_deb_requirements(build_command, first_app_config):
    """Debian requirements can be verified."""
    first_app_config.target_vendor_base = "debian"

    build_command.verify_system_packages(first_app_config)

    # The packages were verified
    assert build_command.tools.subprocess.check_output.mock_calls == [
        call(["dpkg", "-s", "python3-dev"]),
        call(["dpkg", "-s", "build-essential"]),
    ]


def test_rpm_requirements(build_command, first_app_config):
    """RHEL requirements can be verified."""
    first_app_config.target_vendor_base = "rhel"

    build_command.verify_system_packages(first_app_config)

    assert build_command.tools.subprocess.check_output.mock_calls == [
        call(["rpm", "-q", "python3-devel"]),
        call(["rpm", "-q", "gcc"]),
        call(["rpm", "-q", "make"]),
        call(["rpm", "-q", "pkgconf-pkg-config"]),
    ]


def test_suse_requirements(build_command, first_app_config):
    """SUSE requirements can be verified."""
    first_app_config.target_vendor_base = "suse"

    build_command.verify_system_packages(first_app_config)

    assert build_command.tools.subprocess.check_output.mock_calls == [
        call(["rpm", "-q", "--whatprovides", "python3-devel"]),
        call(["rpm", "-q", "--whatprovides", "patterns-devel-base-devel_basis"]),
    ]


def test_arch_requirements(build_command, first_app_config, capsys):
    """Arch requirements can be verified."""
    first_app_config.target_vendor_base = "arch"

    build_command.verify_system_packages(first_app_config)

    assert build_command.tools.subprocess.check_output.mock_calls == [
        call(["pacman", "-Q", "python3"]),
        call(["pacman", "-Q", "base-devel"]),
    ]


def test_unknown_requirements(build_command, first_app_config, capsys):
    """An unknown system can't be verified."""
    first_app_config.target_vendor_base = "somevendor"

    build_command.verify_system_packages(first_app_config)

    # No packages verified
    build_command.tools.subprocess.check_output.assert_not_called()

    # A warning was logged.
    output = capsys.readouterr().out
    assert "WARNING: Can't verify system packages" in output


def test_missing_packages(build_command, first_app_config, capsys):
    """If there are missing system packages, an error is raised."""
    # Mock the system requirement tools; there's a base requirement of
    # a packaged called "compiler", verified using "check <pkg>", and
    # installed using "system <pkg>"
    build_command._system_requirement_tools = MagicMock(
        return_value=(
            ["compiler"],
            ["check"],
            ["system", "install_flag"],
        )
    )

    # Add some system requirements.
    first_app_config.system_requires = ["first", "second", "third"]

    # Mock the side effect of checking those requirements.
    build_command.tools.subprocess.check_output.side_effect = [
        subprocess.CalledProcessError(cmd="check", returncode=1),
        "installed",
        subprocess.CalledProcessError(cmd="check", returncode=1),
        "installed",
    ]

    # Verify the requirements. This will raise an error, but the error
    # message will tell you how to install the system packages.
    with pytest.raises(
        BriefcaseCommandError,
        match=r"    sudo system install_flag compiler second",
    ):
        build_command.verify_system_packages(first_app_config)


def test_missing_system_verify(build_command, first_app_config, capsys):
    """If the program to verify system packages doesn't exist, a warning is logged."""
    # Mock the system verifier is missing
    build_command.tools.shutil.which = MagicMock(return_value="")

    build_command.verify_system_packages(first_app_config)

    # No packages verified
    build_command.tools.subprocess.check_output.assert_not_called()

    # A warning was logged.
    output = capsys.readouterr().out
    assert "WARNING: Can't verify system packages" in output


def test_packages_installed(build_command, first_app_config, capsys):
    """If all required packages are installed, no error is raised."""
    # Mock the system requirement tools; there's a base requirement of
    # a packaged called "compiler", verified using "check <pkg>", and
    # installed using "system <pkg>"
    build_command._system_requirement_tools = MagicMock(
        return_value=(
            ["compiler"],
            ["check"],
            ["system", "install_flag"],
        )
    )

    # Add some system requirements.
    first_app_config.system_requires = ["first", "second", "third"]

    # Mock the effect of checking requirements that are all present
    build_command.tools.subprocess.check_output.return_value = "installed"

    # Verify the requirements. This will raise an error.
    build_command.verify_system_packages(first_app_config)

    assert build_command.tools.subprocess.check_output.mock_calls == [
        call(["check", "compiler"]),
        call(["check", "first"]),
        call(["check", "second"]),
        call(["check", "third"]),
    ]



import os
import shutil
from unittest.mock import MagicMock

import pytest

from briefcase.exceptions import (
    BriefcaseCommandError,
    MissingToolError,
    NetworkFailure,
    NonManagedToolError,
)
from briefcase.integrations.base import ToolCache
from briefcase.integrations.java import JDK

from ...utils import create_zip_file
from .conftest import JDK_BUILD, JDK_RELEASE


@pytest.fixture
def mock_tools(mock_tools) -> ToolCache:
    mock_tools.host_os = "Linux"
    mock_tools.host_arch = "x86_64"
    return mock_tools


def test_non_managed_install(mock_tools, tmp_path, capsys):
    """If the Java install points to a non-managed install, no upgrade is attempted."""

    # Make the installation point to somewhere else.
    jdk = JDK(mock_tools, java_home=tmp_path / "other-jdk")

    # Attempt an upgrade. This will fail because the install is non-managed
    with pytest.raises(NonManagedToolError):
        jdk.upgrade()

    # No download was attempted
    assert mock_tools.file.download.call_count == 0


def test_non_existing_install(mock_tools, tmp_path):
    """If there's no existing managed JDK install, upgrading is an error."""
    # Create an SDK wrapper around a non-existing managed install
    jdk = JDK(mock_tools, java_home=tmp_path / "tools/java")

    with pytest.raises(MissingToolError):
        jdk.upgrade()

    # No download was attempted
    assert mock_tools.file.download.call_count == 0


def test_existing_install(mock_tools, tmp_path):
    """If there's an existing managed JDK install, it is deleted and re-downloaded."""
    # Create a mock of a previously installed Java version.
    java_home = tmp_path / "tools/java"
    (java_home / "bin").mkdir(parents=True)

    # We actually need to delete the original java install
    def rmtree(path):
        shutil.rmtree(path)

    mock_tools.shutil.rmtree.side_effect = rmtree

    # Mock the cached download path.
    jdk_zip_path = create_zip_file(tmp_path / "download.zip", content=[("jdk", "jdk")])
    mock_tools.file.download = MagicMock(return_value=jdk_zip_path)

    # Create a directory to make it look like Java was downloaded and unpacked.
    (tmp_path / "tools" / f"jdk-{JDK_RELEASE}+{JDK_BUILD}").mkdir(parents=True)

    # Create an SDK wrapper
    jdk = JDK(mock_tools, java_home=java_home)

    # Attempt an upgrade.
    jdk.upgrade()

    # The old version has been deleted
    mock_tools.shutil.rmtree.assert_called_with(java_home)

    # A download was initiated
    mock_tools.file.download.assert_called_with(
        url="https://github.com/adoptium/temurin17-binaries/releases/download/"
        f"jdk-{JDK_RELEASE}+{JDK_BUILD}/OpenJDK17U-jdk_x64_linux_hotspot_{JDK_RELEASE}_{JDK_BUILD}.tar.gz",
        download_path=tmp_path / "tools",
        role="Java 17 JDK",
    )

    # The archive was unpacked.
    mock_tools.shutil.unpack_archive.assert_called_with(
        filename=os.fsdecode(jdk_zip_path), extract_dir=os.fsdecode(tmp_path / "tools")
    )
    # The original archive was deleted
    assert not jdk_zip_path.exists()


def test_macOS_existing_install(mock_tools, tmp_path):
    """If there's an existing managed macOS JDK install, it is deleted and re-
    downloaded."""
    # Force mocking on macOS
    mock_tools.host_os = "Darwin"

    # Create a mock of a previously installed Java version.
    java_home = tmp_path / "tools/java/Contents/Home"
    (java_home / "bin").mkdir(parents=True)

    # We actually need to delete the original java install
    def rmtree(path):
        shutil.rmtree(path)

    mock_tools.shutil.rmtree.side_effect = rmtree

    # Mock the cached download path.
    jdk_zip_path = create_zip_file(tmp_path / "download.zip", content=[("jdk", "jdk")])
    mock_tools.file.download = MagicMock(return_value=jdk_zip_path)

    # Create a directory to make it look like Java was downloaded and unpacked.
    (tmp_path / "tools" / f"jdk-{JDK_RELEASE}+{JDK_BUILD}").mkdir(parents=True)

    # Create an SDK wrapper
    jdk = JDK(mock_tools, java_home=java_home)

    # Attempt an upgrade.
    jdk.upgrade()

    # The old version has been deleted
    mock_tools.shutil.rmtree.assert_called_with(tmp_path / "tools/java")

    # A download was initiated
    mock_tools.file.download.assert_called_with(
        url="https://github.com/adoptium/temurin17-binaries/releases/download/"
        f"jdk-{JDK_RELEASE}+{JDK_BUILD}/OpenJDK17U-jdk_x64_mac_hotspot_{JDK_RELEASE}_{JDK_BUILD}.tar.gz",
        download_path=tmp_path / "tools",
        role="Java 17 JDK",
    )

    # The archive was unpacked.
    mock_tools.shutil.unpack_archive.assert_called_with(
        filename=os.fsdecode(jdk_zip_path),
        extract_dir=os.fsdecode(tmp_path / "tools"),
    )

    # The original archive was deleted
    assert not jdk_zip_path.exists()


def test_download_fail(mock_tools, tmp_path):
    """If there's an existing managed JDK install, it is deleted and re-downloaded."""
    # Create a mock of a previously installed Java version.
    java_home = tmp_path / "tools/java"
    (java_home / "bin").mkdir(parents=True)

    # We actually need to delete the original java install
    def rmtree(path):
        shutil.rmtree(path)

    mock_tools.shutil.rmtree.side_effect = rmtree

    # Mock a failure on download
    mock_tools.file.download = MagicMock(side_effect=NetworkFailure("mock"))

    # Create an SDK wrapper
    jdk = JDK(mock_tools, java_home=java_home)

    # Attempt an upgrade. This will fail along with the download
    with pytest.raises(NetworkFailure, match="Unable to mock"):
        jdk.upgrade()

    # The old version has been deleted
    mock_tools.shutil.rmtree.assert_called_with(java_home)

    # A download was initiated
    mock_tools.file.download.assert_called_with(
        url="https://github.com/adoptium/temurin17-binaries/releases/download/"
        f"jdk-{JDK_RELEASE}+{JDK_BUILD}/OpenJDK17U-jdk_x64_linux_hotspot_{JDK_RELEASE}_{JDK_BUILD}.tar.gz",
        download_path=tmp_path / "tools",
        role="Java 17 JDK",
    )

    # No attempt was made to unpack the archive
    assert mock_tools.shutil.unpack_archive.call_count == 0


def test_unpack_fail(mock_tools, tmp_path):
    """If there's an existing managed JDK install, it is deleted and re-downloaded."""
    # Create a mock of a previously installed Java version.
    java_home = tmp_path / "tools/java"
    (java_home / "bin").mkdir(parents=True)

    # We actually need to delete the original java install
    def rmtree(path):
        shutil.rmtree(path)

    mock_tools.shutil.rmtree.side_effect = rmtree

    # Mock the cached download path
    jdk_zip_path = create_zip_file(tmp_path / "download.zip", content=[("jdk", "jdk")])
    mock_tools.file.download = MagicMock(return_value=jdk_zip_path)

    # Mock an unpack failure due to an invalid archive
    mock_tools.shutil.unpack_archive.side_effect = shutil.ReadError

    # Create an SDK wrapper
    jdk = JDK(mock_tools, java_home=java_home)

    # Attempt an upgrade. This will fail.
    with pytest.raises(BriefcaseCommandError):
        jdk.upgrade()

    # The old version has been deleted
    mock_tools.shutil.rmtree.assert_called_with(java_home)

    # A download was initiated
    mock_tools.file.download.assert_called_with(
        url="https://github.com/adoptium/temurin17-binaries/releases/download/"
        f"jdk-{JDK_RELEASE}+{JDK_BUILD}/OpenJDK17U-jdk_x64_linux_hotspot_{JDK_RELEASE}_{JDK_BUILD}.tar.gz",
        download_path=tmp_path / "tools",
        role="Java 17 JDK",
    )

    # The archive was unpacked.
    mock_tools.shutil.unpack_archive.assert_called_with(
        filename=os.fsdecode(jdk_zip_path),
        extract_dir=os.fsdecode(tmp_path / "tools"),
    )
    # The original archive was not deleted
    assert jdk_zip_path.exists()



import os
import subprocess
from unittest import mock
from unittest.mock import MagicMock
from zipfile import ZipFile

import pytest

from briefcase.console import Console, Log
from briefcase.exceptions import BriefcaseCommandError
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.macOS.app import macOSAppPackageCommand


@pytest.fixture
def package_command(tmp_path):
    command = macOSAppPackageCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.os = MagicMock(spec_set=os)

    command.tools.subprocess = MagicMock(spec_set=Subprocess)

    return command


@pytest.fixture
def first_app_dmg(tmp_path):
    dmg_path = tmp_path / "base_path/dist/First App.dmg"
    dmg_path.parent.mkdir(parents=True)
    with dmg_path.open("w", encoding="utf-8") as f:
        f.write("DMG content here")

    return dmg_path


@pytest.fixture
def first_app_pkg(tmp_path):
    dmg_path = tmp_path / "base_path/dist/First App-0.0.1.pkg"
    dmg_path.parent.mkdir(parents=True)
    with dmg_path.open("w", encoding="utf-8") as f:
        f.write("PKG content here")

    return dmg_path


def test_notarize_app(
    package_command,
    first_app_with_binaries,
    sekrit_identity,
    tmp_path,
):
    """An app can be notarized."""
    app_path = (
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "macos"
        / "app"
        / "First App.app"
    )
    archive_path = tmp_path / "base_path/build/first-app/macos/app/archive.zip"
    package_command.notarize(app_path, identity=sekrit_identity)

    # As a result of mocking os.unlink, the zip archive won't be
    # cleaned up, so we can test for its existence, but also
    # verify that it *would* have been deleted.
    assert archive_path.exists()
    package_command.tools.os.unlink.assert_called_with(archive_path)
    # The archive contains the app as the only top level element.
    with ZipFile(archive_path) as archive:
        assert sorted(archive.namelist()) == [
            "First App.app/",
            "First App.app/Contents/",
            "First App.app/Contents/Frameworks/",
            "First App.app/Contents/Frameworks/Extras.framework/",
            "First App.app/Contents/Frameworks/Extras.framework/Resources/",
            "First App.app/Contents/Frameworks/Extras.framework/Resources/extras.dylib",
            "First App.app/Contents/Info.plist",
            "First App.app/Contents/MacOS/",
            "First App.app/Contents/MacOS/First App",
            "First App.app/Contents/Resources/",
            "First App.app/Contents/Resources/app_packages/",
            "First App.app/Contents/Resources/app_packages/Extras.app/",
            "First App.app/Contents/Resources/app_packages/Extras.app/Contents/",
            "First App.app/Contents/Resources/app_packages/Extras.app/Contents/MacOS/",
            "First App.app/Contents/Resources/app_packages/Extras.app/Contents/MacOS/Extras",
            "First App.app/Contents/Resources/app_packages/first.other",
            "First App.app/Contents/Resources/app_packages/first_dylib.dylib",
            "First App.app/Contents/Resources/app_packages/first_so.so",
            "First App.app/Contents/Resources/app_packages/other_binary",
            "First App.app/Contents/Resources/app_packages/second.other",
            "First App.app/Contents/Resources/app_packages/special.binary",
            "First App.app/Contents/Resources/app_packages/subfolder/",
            "First App.app/Contents/Resources/app_packages/subfolder/second_dylib.dylib",
            "First App.app/Contents/Resources/app_packages/subfolder/second_so.so",
            "First App.app/Contents/Resources/app_packages/unknown.binary",
        ]

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit *archive* for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    archive_path,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
            # Staple the result to the *app*
            mock.call(
                [
                    "xcrun",
                    "stapler",
                    "staple",
                    app_path,
                ],
                check=True,
            ),
        ]
    )


def test_notarize_dmg(
    package_command,
    first_app_dmg,
    sekrit_identity,
):
    """A DMG can be notarized."""

    package_command.notarize(first_app_dmg, identity=sekrit_identity)

    # The DMG didn't require an archive file, so unlink wasn't invoked.
    package_command.tools.os.unlink.assert_not_called()

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    first_app_dmg,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
            # Staple the result
            mock.call(
                [
                    "xcrun",
                    "stapler",
                    "staple",
                    first_app_dmg,
                ],
                check=True,
            ),
        ]
    )


def test_notarize_pkg(
    package_command,
    first_app_pkg,
    sekrit_identity,
):
    """A PKG can be notarized."""

    package_command.notarize(first_app_pkg, identity=sekrit_identity)

    # The PKG didn't require an archive file, so unlink wasn't invoked.
    package_command.tools.os.unlink.assert_not_called()

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    first_app_pkg,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
            # Staple the result
            mock.call(
                [
                    "xcrun",
                    "stapler",
                    "staple",
                    first_app_pkg,
                ],
                check=True,
            ),
        ]
    )


def test_notarize_unknown_format(package_command, sekrit_identity, tmp_path):
    """Attempting to notarize a file of unknown format raises an error."""
    pkg_path = tmp_path / "base_path/dist/First App.foo"

    # The notarization call will fail with an error
    with pytest.raises(
        RuntimeError,
        match=r"Don't know how to notarize a file of type .foo",
    ):
        package_command.notarize(pkg_path, identity=sekrit_identity)


def test_notarize_unknown_credentials(
    package_command,
    first_app_dmg,
    sekrit_identity,
):
    """When notarizing, if credentials haven't been stored, the user will be prompted to
    store them."""
    # Set up subprocess to fail on the first notarization attempt
    package_command.tools.subprocess.run.side_effect = [
        subprocess.CalledProcessError(
            returncode=69,
            cmd=["xcrun", "notarytool", "submit"],
        ),  # Unknown credential failure
        None,  # Successful credential storage
        None,  # Successful notarization
        None,  # Successful stapling
    ]

    package_command.notarize(first_app_dmg, identity=sekrit_identity)

    # The DMG didn't require an archive file, so unlink wasn't invoked.
    package_command.tools.os.unlink.assert_not_called()

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    first_app_dmg,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
            # Store credentials in the keychain
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "store-credentials",
                    "--team-id",
                    "DEADBEEF",
                    "briefcase-macOS-DEADBEEF",
                ],
                check=True,
                stream_output=False,
            ),
            # Submit for notarization a second time
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    first_app_dmg,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
            # Staple the result
            mock.call(
                [
                    "xcrun",
                    "stapler",
                    "staple",
                    first_app_dmg,
                ],
                check=True,
            ),
        ]
    )


def test_credential_storage_failure_app(
    package_command,
    first_app_with_binaries,
    sekrit_identity,
    tmp_path,
):
    """When submitting an app, if credentials haven't been stored, and storage fails, an
    error is raised."""
    app_path = (
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "macos"
        / "app"
        / "First App.app"
    )
    archive_path = tmp_path / "base_path/build/first-app/macos/app/archive.zip"

    # Set up subprocess to fail on the first notarization attempt,
    # then fail on the storage of credentials
    package_command.tools.subprocess.run.side_effect = [
        subprocess.CalledProcessError(
            returncode=69,
            cmd=["xcrun", "notarytool", "submit"],
        ),  # Unknown credential failure
        subprocess.CalledProcessError(
            returncode=1,
            cmd=["xcrun", "notarytool", "store-credentials"],
        ),  # Credential verification failed
    ]

    # The notarization call will fail with an error
    with pytest.raises(
        BriefcaseCommandError,
        match=r"Unable to store credentials for team ID DEADBEEF.",
    ):
        package_command.notarize(app_path, identity=sekrit_identity)

    # As a result of mocking os.unlink, the zip archive won't be
    # cleaned up, so we can test for its existence, but also
    # verify that it *would* have been deleted.
    assert archive_path.exists()
    package_command.tools.os.unlink.assert_called_with(archive_path)

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    archive_path,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
            # Store credentials in the keychain
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "store-credentials",
                    "--team-id",
                    "DEADBEEF",
                    "briefcase-macOS-DEADBEEF",
                ],
                check=True,
                stream_output=False,
            ),
        ]
    )


def test_credential_storage_failure_dmg(
    package_command,
    first_app_dmg,
    sekrit_identity,
):
    """If credentials haven't been stored, and storage fails, an error is raised."""
    # Set up subprocess to fail on the first notarization attempt,
    # then fail on the storage of credentials
    package_command.tools.subprocess.run.side_effect = [
        subprocess.CalledProcessError(
            returncode=69,
            cmd=["xcrun", "notarytool", "submit"],
        ),  # Unknown credential failure
        subprocess.CalledProcessError(
            returncode=1,
            cmd=["xcrun", "notarytool", "store-credentials"],
        ),  # Credential verification failed
    ]

    # The notarization call will fail with an error
    with pytest.raises(
        BriefcaseCommandError,
        match=r"Unable to store credentials for team ID DEADBEEF.",
    ):
        package_command.notarize(first_app_dmg, identity=sekrit_identity)

    # The DMG didn't require an archive file, so unlink wasn't invoked.
    package_command.tools.os.unlink.assert_not_called()

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    first_app_dmg,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
            # Store credentials in the keychain
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "store-credentials",
                    "--team-id",
                    "DEADBEEF",
                    "briefcase-macOS-DEADBEEF",
                ],
                check=True,
                stream_output=False,
            ),
        ]
    )


def test_credential_storage_disabled_input_app(
    package_command,
    first_app_with_binaries,
    sekrit_identity,
    tmp_path,
):
    """When packaging an app, if credentials haven't been stored, and input is disabled,
    an error is raised."""
    app_path = (
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "macos"
        / "app"
        / "First App.app"
    )
    archive_path = tmp_path / "base_path/build/first-app/macos/app/archive.zip"

    # Set up subprocess to fail on the first notarization attempt.
    package_command.tools.subprocess.run.side_effect = [
        subprocess.CalledProcessError(
            returncode=69,
            cmd=["xcrun", "notarytool", "submit"],
        ),  # Unknown credential failure
    ]
    # Disable console input
    package_command.tools.input.enabled = False

    # The notarization call will fail with an error
    with pytest.raises(
        BriefcaseCommandError,
        match=r"The keychain does not contain credentials for the profile briefcase-macOS-DEADBEEF.",
    ):
        package_command.notarize(app_path, identity=sekrit_identity)

    # As a result of mocking os.unlink, the zip archive won't be
    # cleaned up, so we can test for its existence, but also
    # verify that it *would* have been deleted.
    assert archive_path.exists()
    package_command.tools.os.unlink.assert_called_with(archive_path)

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    archive_path,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
        ]
    )


def test_credential_storage_disabled_input_dmg(
    package_command,
    first_app_dmg,
    sekrit_identity,
):
    """When packaging a DMG, if credentials haven't been stored, and input is disabled,
    an error is raised."""
    # Set up subprocess to fail on the first notarization attempt.
    package_command.tools.subprocess.run.side_effect = [
        subprocess.CalledProcessError(
            returncode=69,
            cmd=["xcrun", "notarytool", "submit"],
        ),  # Unknown credential failure
    ]
    # Disable console input
    package_command.tools.input.enabled = False

    # The notarization call will fail with an error
    with pytest.raises(
        BriefcaseCommandError,
        match=r"The keychain does not contain credentials for the profile briefcase-macOS-DEADBEEF.",
    ):
        package_command.notarize(first_app_dmg, identity=sekrit_identity)

    # The DMG didn't require an archive file, so unlink wasn't invoked.
    package_command.tools.os.unlink.assert_not_called()

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    first_app_dmg,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
        ]
    )


def test_notarize_unknown_credentials_after_storage(
    package_command,
    first_app_dmg,
    sekrit_identity,
):
    """If we get a credential failure after an attempt to store, an error is raised."""
    # Set up subprocess to fail on the second notarization attempt
    package_command.tools.subprocess.run.side_effect = [
        subprocess.CalledProcessError(
            returncode=69,
            cmd=["xcrun", "notarytool", "submit"],
        ),  # Unknown credential failure
        None,  # Successful credential storage
        subprocess.CalledProcessError(
            returncode=69,
            cmd=["xcrun", "notarytool", "submit"],
        ),  # A second unknown credential failure
        None,  # Successful stapling
    ]

    # The notarization call will fail with an error
    with pytest.raises(
        BriefcaseCommandError,
        match=r"Unable to submit dist[/\\]First App.dmg for notarization.",
    ):
        package_command.notarize(first_app_dmg, identity=sekrit_identity)

    # The DMG didn't require an archive file, so unlink wasn't invoked.
    package_command.tools.os.unlink.assert_not_called()

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    first_app_dmg,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
            # Store credentials in the keychain
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "store-credentials",
                    "--team-id",
                    "DEADBEEF",
                    "briefcase-macOS-DEADBEEF",
                ],
                check=True,
                stream_output=False,
            ),
            # Submit for notarization a second time
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    first_app_dmg,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
        ]
    )


def test_app_notarization_failure_with_credentials(
    package_command,
    first_app_with_binaries,
    sekrit_identity,
    tmp_path,
):
    """If the notarization process for an app fails for a reason other than credentials,
    an error is raised."""
    app_path = (
        tmp_path
        / "base_path"
        / "build"
        / "first-app"
        / "macos"
        / "app"
        / "First App.app"
    )
    archive_path = tmp_path / "base_path/build/first-app/macos/app/archive.zip"

    # Set up subprocess to fail on the first notarization attempt
    # for a reason other than credentials
    package_command.tools.subprocess.run.side_effect = [
        subprocess.CalledProcessError(
            returncode=42,
            cmd=["xcrun", "notarytool", "submit"],
        ),  # Notarization failure; error code 42 is a fake value
    ]

    # The notarization call will fail with an error
    with pytest.raises(
        BriefcaseCommandError,
        match=r"Unable to submit build[/\\]first-app[/\\]macos[/\\]app[/\\]First App.app for notarization.",
    ):
        package_command.notarize(app_path, identity=sekrit_identity)

    # As a result of mocking os.unlink, the zip archive won't be
    # cleaned up, so we can test for its existence, but also
    # verify that it *would* have been deleted.
    assert archive_path.exists()
    package_command.tools.os.unlink.assert_called_with(archive_path)

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    archive_path,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
        ]
    )


def test_dmg_notarization_failure_with_credentials(
    package_command,
    first_app_dmg,
    sekrit_identity,
):
    """If the notarization process for a DMG fails for a reason other than credentials,
    an error is raised."""
    # Set up subprocess to fail on the first notarization attempt
    # for a reason other than credentials
    package_command.tools.subprocess.run.side_effect = [
        subprocess.CalledProcessError(
            returncode=42,
            cmd=["xcrun", "notarytool", "submit"],
        ),  # Notarization failure; error code 42 is a fake value
    ]

    # The notarization call will fail with an error
    with pytest.raises(
        BriefcaseCommandError,
        match=r"Unable to submit dist[/\\]First App.dmg for notarization.",
    ):
        package_command.notarize(first_app_dmg, identity=sekrit_identity)

    # The DMG didn't require an archive file, so unlink wasn't invoked.
    package_command.tools.os.unlink.assert_not_called()

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    first_app_dmg,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
        ]
    )


def test_stapling_failure(
    package_command,
    first_app_dmg,
    sekrit_identity,
):
    """If the stapling process fails, an error is raised."""
    # Set up a failure in the stapling process
    package_command.tools.subprocess.run.side_effect = [
        None,
        subprocess.CalledProcessError(
            returncode=42,
            cmd=["xcrun", "stapler"],
        ),  # Stapling failure; error code 42 is a fake value
    ]

    with pytest.raises(
        BriefcaseCommandError,
        match=r"Unable to staple notarization onto dist[/\\]First App.dmg",
    ):
        package_command.notarize(first_app_dmg, identity=sekrit_identity)

    # The DMG didn't require an archive file, so unlink wasn't invoked.
    package_command.tools.os.unlink.assert_not_called()

    # The calls to notarize were made
    package_command.tools.subprocess.run.assert_has_calls(
        [
            # Submit for notarization
            mock.call(
                [
                    "xcrun",
                    "notarytool",
                    "submit",
                    first_app_dmg,
                    "--keychain-profile",
                    "briefcase-macOS-DEADBEEF",
                    "--wait",
                ],
                check=True,
            ),
            # Staple the result
            mock.call(
                [
                    "xcrun",
                    "stapler",
                    "staple",
                    first_app_dmg,
                ],
                check=True,
            ),
        ]
    )



import pytest

from briefcase.integrations.linuxdeploy import LinuxDeployGtkPlugin

from ...utils import assert_url_resolvable


@pytest.fixture
def linuxdeploy_gtk_plugin(mock_tools):
    return LinuxDeployGtkPlugin(mock_tools)


def test_file_path(linuxdeploy_gtk_plugin, mock_tools):
    """Default Linuxdeploy plugins reside in the linuxdeploy plugins path."""
    assert (
        linuxdeploy_gtk_plugin.file_path
        == mock_tools.base_path / "linuxdeploy_plugins/gtk"
    )


def test_file_name(linuxdeploy_gtk_plugin):
    """GTK plugin has a known name."""
    assert linuxdeploy_gtk_plugin.file_name == "linuxdeploy-plugin-gtk.sh"


def test_plugin_id(linuxdeploy_gtk_plugin):
    """GTK plugin ID is fixed."""
    assert linuxdeploy_gtk_plugin.plugin_id == "gtk"


def test_download_url(linuxdeploy_gtk_plugin):
    """GTK plugin download URL is architecture dependent."""
    assert linuxdeploy_gtk_plugin.download_url == (
        "https://raw.githubusercontent.com/linuxdeploy/linuxdeploy-plugin-gtk/"
        "master/linuxdeploy-plugin-gtk.sh"
    )
    assert_url_resolvable(linuxdeploy_gtk_plugin.download_url)



from pathlib import Path
from unittest.mock import MagicMock

import pytest

from briefcase.integrations.android_sdk import ADB, AndroidSDK
from briefcase.integrations.base import ToolCache
from briefcase.integrations.file import File
from briefcase.integrations.java import JDK
from briefcase.integrations.subprocess import Subprocess

# current versions of Android SDK Manager
SDK_MGR_VER = "12.0"
SDK_MGR_DL_VER = "11076708"


@pytest.fixture
def mock_tools(mock_tools, tmp_path) -> ToolCache:
    # Mock default tools
    mock_tools.subprocess = MagicMock(spec_set=Subprocess)
    mock_tools.file.download = MagicMock(spec_set=File.download)

    # Set up a JDK
    mock_tools.java = MagicMock(spec=JDK)
    mock_tools.java.java_home = Path("/path/to/jdk")

    return mock_tools


@pytest.fixture
def android_sdk(mock_tools, tmp_path) -> AndroidSDK:
    # Ensure root directory for SDK exists
    sdk_root = tmp_path / "sdk"
    sdk_root.mkdir(parents=True)

    return AndroidSDK(mock_tools, root_path=sdk_root)


@pytest.fixture
def adb(mock_tools) -> ADB:
    return ADB(mock_tools, "exampleDevice")



import logging

import pytest

from briefcase.console import LogLevel, RichLoggingHandler

cookiecutter_logger = logging.getLogger("cookiecutter")


@pytest.fixture
def base_command(base_command):
    # Mock actual templating commands as no-ops
    base_command.update_cookiecutter_cache = lambda *a, **kw: None
    base_command.tools.cookiecutter = lambda *a, **kw: None
    return base_command


@pytest.mark.parametrize(
    "logging_level, handler_expected",
    [
        (LogLevel.DEEP_DEBUG, True),
        (LogLevel.DEBUG, False),
        (LogLevel.VERBOSE, False),
        (LogLevel.INFO, False),
    ],
)
def test_git_stdlib_logging(base_command, logging_level, handler_expected):
    """A logging handler is configured for GitPython when DEEP_DEBUG is enabled."""
    base_command.logger.verbosity = logging_level

    base_command.generate_template(
        template="",
        branch="",
        output_path="",
        extra_context={},
    )

    assert handler_expected is any(
        isinstance(h, RichLoggingHandler)
        for h in logging.getLogger("cookiecutter").handlers
    )

    # reset handlers since they are persistent
    logging.getLogger("cookiecutter").handlers.clear()



import os
import shutil
import sys
from unittest import mock

import pytest
from requests import exceptions as requests_exceptions

from briefcase.exceptions import (
    InvalidStubBinary,
    MissingNetworkResourceError,
    MissingStubBinary,
    NetworkFailure,
)

from ...utils import create_file, create_tgz_file, create_zip_file, mock_zip_download


@pytest.mark.parametrize("console_app", [True, False])
def test_install_stub_binary(
    create_command,
    myapp,
    console_app,
    stub_binary_revision_path_index,
    tmp_path,
):
    """A stub binary can be downloaded and unpacked where it is needed."""
    # Mock the app type
    myapp.console_app = console_app
    stub_name = "Console-Stub" if console_app else "GUI-Stub"

    # Mock download.file to return a stub binary
    create_command.tools.file.download = mock.MagicMock(
        side_effect=mock_zip_download(
            f"{stub_name}-3.X-b37.zip",
            [("Stub.bin", "stub binary")],
        )
    )

    # Wrap shutil so we can confirm that unpack is called
    create_command.tools.shutil = mock.MagicMock(wraps=shutil)

    # Install the stub binary
    create_command.install_stub_binary(myapp)

    # Confirm the right URL was used
    create_command.tools.file.download.assert_called_with(
        download_path=create_command.data_path / "stub",
        url=f"https://briefcase-support.s3.amazonaws.com/python/3.X/Tester/{stub_name}-3.X-b37.zip",
        role="stub binary",
    )

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=tmp_path / f"data/stub/{stub_name}-3.X-b37.zip",
        extract_dir=tmp_path / "base_path/build/my-app/tester/dummy",
    )

    # Confirm that the full path to the stub file has been unpacked.
    assert (tmp_path / "base_path/build/my-app/tester/dummy/Stub.bin").exists()


@pytest.mark.parametrize("console_app", [True, False])
def test_install_stub_binary_unpack_failure(
    create_command,
    myapp,
    console_app,
    stub_binary_revision_path_index,
    tmp_path,
):
    """Errors during unpacking the archive raise InvalidStubBinary."""
    # Mock the app type
    myapp.console_app = console_app
    stub_name = "Console-Stub" if console_app else "GUI-Stub"

    # Mock download.file to return a stub binary
    create_command.tools.file.download = mock.MagicMock(
        side_effect=mock_zip_download(
            f"{stub_name}-3.X-b37.zip",
            [("Stub.bin", "stub binary")],
        )
    )

    # Wrap shutil so we can confirm that unpack is called
    create_command.tools.shutil = mock.MagicMock(wraps=shutil)
    create_command.tools.shutil.unpack_archive.side_effect = shutil.ReadError

    # Install the stub binary
    with pytest.raises(InvalidStubBinary, match="Unable to unpack or copy stub binary"):
        create_command.install_stub_binary(myapp)

    # Confirm the right URL was used
    create_command.tools.file.download.assert_called_with(
        download_path=create_command.data_path / "stub",
        url=f"https://briefcase-support.s3.amazonaws.com/python/3.X/Tester/{stub_name}-3.X-b37.zip",
        role="stub binary",
    )

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=tmp_path / f"data/stub/{stub_name}-3.X-b37.zip",
        extract_dir=tmp_path / "base_path/build/my-app/tester/dummy",
    )


@pytest.mark.parametrize("console_app", [True, False])
def test_install_pinned_stub_binary(
    create_command,
    myapp,
    console_app,
    stub_binary_revision_path_index,
    tmp_path,
):
    """The stub binary revision can be pinned."""
    # Pin the stub binary revision
    myapp.stub_binary_revision = 42

    # Mock the app type
    myapp.console_app = console_app
    stub_name = "Console-Stub" if console_app else "GUI-Stub"

    # Mock download.file to return a stub binary
    create_command.tools.file.download = mock.MagicMock(
        side_effect=mock_zip_download(
            f"{stub_name}-3.X-b42.zip",
            [("Stub.bin", "stub binary")],
        )
    )

    # Wrap shutil so we can confirm that unpack is called
    create_command.tools.shutil = mock.MagicMock(wraps=shutil)

    # Install the stub binary
    create_command.install_stub_binary(myapp)

    # Confirm the right URL was used
    create_command.tools.file.download.assert_called_with(
        download_path=create_command.data_path / "stub",
        url=f"https://briefcase-support.s3.amazonaws.com/python/3.X/Tester/{stub_name}-3.X-b42.zip",
        role="stub binary",
    )

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=tmp_path / f"data/stub/{stub_name}-3.X-b42.zip",
        extract_dir=tmp_path / "base_path/build/my-app/tester/dummy",
    )

    # Confirm that the full path to the stub file has been unpacked.
    assert (tmp_path / "base_path/build/my-app/tester/dummy/Stub.bin").exists()


def test_install_stub_binary_missing(
    create_command,
    myapp,
    stub_binary_revision_path_index,
    tmp_path,
):
    """If the system-nominated stub binary doesn't exist, a specific error is raised."""
    # Modify download.file to raise an exception
    create_command.tools.file.download = mock.MagicMock(
        side_effect=MissingNetworkResourceError(
            "https://briefcase-support.s3.amazonaws.com/python/3.X/Tester/GUI-Stub-3.X-b37.zip"
        )
    )

    # Install the stub binary; this will raise a custom exception
    with pytest.raises(
        MissingStubBinary,
        match=r"Unable to download Tester stub binary for Python 3.X on gothic",
    ):
        create_command.install_stub_binary(myapp)


def test_install_custom_stub_binary_url(
    create_command,
    myapp,
    stub_binary_revision_path_index,
    tmp_path,
):
    """A stub binary can be downloaded and unpacked where it is needed."""
    # Provide an app-specific override of the stub binary as a URL
    myapp.stub_binary = "https://example.com/custom/My-Stub.zip"

    # Mock download.file to return a stub binary
    create_command.tools.file.download = mock.MagicMock(
        side_effect=mock_zip_download(
            "My-Stub.zip",
            [("Stub.bin", "stub binary")],
        )
    )

    # Wrap shutil so we can confirm that unpack is called
    create_command.tools.shutil = mock.MagicMock(wraps=shutil)

    # Install the stub binary
    create_command.install_stub_binary(myapp)

    # Confirm the right URL was used
    create_command.tools.file.download.assert_called_with(
        download_path=create_command.data_path
        / "stub/986428ef9d5a1852fc15d4367f19aa328ad530686056e9d83cdde03407c0bceb",
        url="https://example.com/custom/My-Stub.zip",
        role="stub binary",
    )

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=tmp_path
        / "data/stub/986428ef9d5a1852fc15d4367f19aa328ad530686056e9d83cdde03407c0bceb/My-Stub.zip",
        extract_dir=tmp_path / "base_path/build/my-app/tester/dummy",
    )

    # Confirm that the full path to the stub file has been unpacked.
    assert (tmp_path / "base_path/build/my-app/tester/dummy/Stub.bin").exists()


def test_install_custom_stub_binary_file(
    create_command,
    myapp,
    tmp_path,
    stub_binary_revision_path_index,
):
    """A custom stub binary can be specified as a local file."""
    # Provide an app-specific override of the stub binary
    myapp.stub_binary = os.fsdecode(tmp_path / "custom/My-Stub")

    # Write a temporary stub binary
    create_file(tmp_path / "custom/My-Stub", "Custom stub")

    # Modify download.file to return the temp zipfile
    create_command.tools.file.download = mock.MagicMock()

    # Wrap shutil so we can confirm that unpack is called
    create_command.tools.shutil = mock.MagicMock(wraps=shutil)

    # Install the stub binary
    create_command.install_stub_binary(myapp)

    # There should have been no download attempt,
    # as the resource is local.
    create_command.tools.file.download.assert_not_called()

    # The file isn't an archive, so it hasn't been unpacked.
    create_command.tools.shutil.unpack_archive.assert_not_called()

    # Confirm that the full path to the stub file has been unpacked.
    assert (tmp_path / "base_path/build/my-app/tester/dummy/Stub.bin").exists()


def test_install_custom_stub_binary_zip(
    create_command,
    myapp,
    tmp_path,
    stub_binary_revision_path_index,
):
    """A custom stub binary can be specified as a local archive."""
    # Provide an app-specific override of the stub binary
    myapp.stub_binary = os.fsdecode(tmp_path / "custom/stub.zip")

    # Write a temporary stub zip file
    stub_file = create_zip_file(
        tmp_path / "custom/stub.zip",
        [("Stub.bin", "Custom stub")],
    )

    # Modify download.file to return the temp zipfile
    create_command.tools.file.download = mock.MagicMock()

    # Wrap shutil so we can confirm that unpack is called
    create_command.tools.shutil = mock.MagicMock(wraps=shutil)

    # Install the stub binary
    create_command.install_stub_binary(myapp)

    # There should have been no download attempt,
    # as the resource is local.
    create_command.tools.file.download.assert_not_called()

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=stub_file,
        extract_dir=tmp_path / "base_path/build/my-app/tester/dummy",
    )

    # Confirm that the full path to the stub file has been unpacked.
    assert (tmp_path / "base_path/build/my-app/tester/dummy/Stub.bin").exists()


@pytest.mark.parametrize("stub_filename", ("stub.tar", "stub.tar.gz"))
def test_install_custom_stub_binary_tar(
    create_command,
    myapp,
    stub_filename,
    tmp_path,
    stub_binary_revision_path_index,
):
    """A custom stub binary can be specified as a local archive."""
    # Provide an app-specific override of the stub binary
    myapp.stub_binary = os.fsdecode(tmp_path / f"custom/{stub_filename}")

    # Write a temporary stub zip file
    stub_file = create_tgz_file(
        tmp_path / f"custom/{stub_filename}",
        [("Stub.bin", "Custom stub")],
    )

    # Modify download.file to return the temp zipfile
    create_command.tools.file.download = mock.MagicMock()

    # Wrap shutil so we can confirm that unpack is called
    create_command.tools.shutil = mock.MagicMock(wraps=shutil)

    # Install the stub binary
    create_command.install_stub_binary(myapp)

    # There should have been no download attempt,
    # as the resource is local.
    create_command.tools.file.download.assert_not_called()

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=stub_file,
        extract_dir=tmp_path / "base_path/build/my-app/tester/dummy",
        **({"filter": "data"} if sys.version_info >= (3, 12) else {}),
    )

    # Confirm that the full path to the stub file has been unpacked.
    assert (tmp_path / "base_path/build/my-app/tester/dummy/Stub.bin").exists()


def test_install_custom_stub_binary_with_revision(
    create_command,
    myapp,
    tmp_path,
    stub_binary_revision_path_index,
    capsys,
):
    """If a custom stub binary file also specifies a revision, the revision is ignored
    with a warning."""
    # Provide an app-specific override of the stub binary, *and* the revision
    myapp.stub_binary = os.fsdecode(tmp_path / "custom/stub.zip")
    myapp.stub_binary_revision = "42"

    # Write a temporary stub zip file
    stub_file = create_zip_file(
        tmp_path / "custom/stub.zip",
        [("Stub.bin", "Custom stub")],
    )

    # Modify download.file to return the temp zipfile
    create_command.tools.file.download = mock.MagicMock()

    # Wrap shutil so we can confirm that unpack is called
    create_command.tools.shutil = mock.MagicMock(wraps=shutil)

    # Install the stub binary
    create_command.install_stub_binary(myapp)

    # There should have been no download attempt,
    # as the resource is local.
    create_command.tools.file.download.assert_not_called()

    # Confirm the right file was unpacked
    create_command.tools.shutil.unpack_archive.assert_called_with(
        filename=stub_file,
        extract_dir=tmp_path / "base_path/build/my-app/tester/dummy",
    )

    # Confirm that the full path to the stub file has been unpacked.
    assert (tmp_path / "base_path/build/my-app/tester/dummy/Stub.bin").exists()

    # A warning about the stub revision was generated.
    assert "stub binary revision will be ignored." in capsys.readouterr().out


def test_install_custom_stub_binary_with_invalid_url(
    create_command,
    myapp,
    stub_binary_revision_path_index,
):
    """Invalid URL for a custom stub binary raises MissingNetworkResourceError."""

    # Provide a custom stub binary URL
    url = "https://example.com/custom/stub.zip"
    myapp.stub_binary = url

    # Modify download.file to raise an exception
    create_command.tools.file.download = mock.MagicMock(
        side_effect=MissingNetworkResourceError(url)
    )

    # The bad URL should raise a MissingNetworkResourceError
    with pytest.raises(MissingNetworkResourceError):
        create_command.install_stub_binary(myapp)

    # However, there will have been a download attempt
    create_command.tools.file.download.assert_called_with(
        download_path=(
            create_command.data_path
            / "stub"
            / "8d0f202db2a2c66b1568ead0ca63f66957bd2be7a12145f5b9fa2197a5e049f7"
        ),
        url=url,
        role="stub binary",
    )


def test_offline_install(
    create_command,
    myapp,
    stub_binary_revision_path_index,
):
    """If the computer is offline, an error is raised."""
    create_command.tools.requests.get = mock.MagicMock(
        side_effect=requests_exceptions.ConnectionError
    )

    # Installing while offline raises an error
    with pytest.raises(NetworkFailure):
        create_command.install_stub_binary(myapp)


def test_install_custom_stub_binary_with_invalid_filepath(
    create_command,
    myapp,
    stub_binary_revision_path_index,
    tmp_path,
):
    """Invalid file path for custom stub library raises InvalidStubBinary."""
    # Provide an app-specific override of the stub binary
    myapp.stub_binary = os.fsdecode(tmp_path / "custom/My-Stub")

    # Modify download.file to return the temp zipfile
    create_command.tools.file.download = mock.MagicMock()

    # Mock shutil so we can confirm that unpack isn't called,
    # but we still want the side effect of calling
    create_command.tools.shutil = mock.MagicMock(spec_set=shutil)
    create_command.tools.shutil.copyfile.side_effect = shutil.copyfile

    # Fail to install the stub binary
    with pytest.raises(InvalidStubBinary, match="Unable to unpack or copy stub binary"):
        create_command.install_stub_binary(myapp)

    # There should have been no download attempt,
    # as the resource is local.
    create_command.tools.file.download.assert_not_called()

    # The file isn't an archive, so it hasn't been unpacked.
    create_command.tools.shutil.unpack_archive.assert_not_called()



import sys
from pathlib import Path
from unittest.mock import MagicMock

import pytest

from briefcase.exceptions import BriefcaseCommandError
from briefcase.platforms.linux import system


def test_valid_python3(monkeypatch, create_command):
    """If Briefcase is being run with the system python, verification passes."""

    # Mock the existence of a valid non-docker system Python
    # with the same major/minor as the current Python
    python3 = MagicMock()
    python3.resolve.return_value = Path(
        f"/usr/bin/python{sys.version_info.major}.{sys.version_info.minor}"
    )
    mock_Path = MagicMock(return_value=python3)
    monkeypatch.setattr(system, "Path", mock_Path)

    # System Python can be verified
    create_command.verify_system_python()


@pytest.mark.parametrize(
    "resolved_path, expected_error",
    [
        ("/usr/bin/pythonX", "Can't determine the system python version"),
        ("/usr/bin/python3", "Can't determine the system python version"),
        (
            "/usr/bin/python3.X",
            r"The version of Python being used to run Briefcase \(3\..*\) is not the system python3 \(3.X\)\.",
        ),
    ],
)
def test_bad_python3(monkeypatch, create_command, resolved_path, expected_error):
    """If the system Python3 isn't obviously a Python3, an error is raised."""
    # Mock a Python3 symlink that isn't the existence of a valid non-docker system Python
    # with the same major/minor as the current Python
    python3 = MagicMock()
    python3.resolve.return_value = Path(resolved_path)
    mock_Path = MagicMock(return_value=python3)
    monkeypatch.setattr(system, "Path", mock_Path)

    # Verifying python raises an error
    with pytest.raises(BriefcaseCommandError, match=expected_error):
        create_command.verify_system_python()



from __future__ import annotations

import subprocess
from pathlib import Path

from briefcase.exceptions import BriefcaseCommandError, CommandOutputParseError
from briefcase.integrations.base import Tool, ToolCache
from briefcase.integrations.subprocess import json_parser


class VisualStudio(Tool):
    name = "visualstudio"
    full_name = "Visual Studio"
    supported_host_os = {"Windows"}
    VSCODE_REQUIRED_COMPONENTS = """
    * .NET Desktop Development
      - Default packages
    * Desktop Development with C++
      - Default packages; plus
      - C++/CLI support for v143 build tools
"""

    def __init__(
        self,
        tools: ToolCache,
        msbuild_path: Path,
        install_metadata: dict[str, str | int | bool] = None,
    ):
        super().__init__(tools=tools)
        self._msbuild_path = msbuild_path
        self._install_metadata = install_metadata

    @property
    def msbuild_path(self) -> Path:
        """The path to the MSBuild executable."""
        return self._msbuild_path

    @property
    def install_metadata(self) -> dict[str, str | int | bool] | None:
        """Metadata about the Visual Studio install.

        Will be ``None`` if MSBuild the path to MSBuild has been provided
        explicitly, or is on the path.

        Otherwise, a dictionary containing the install details from VSWhere.
        """
        return self._install_metadata

    @classmethod
    def verify_install(cls, tools: ToolCache, **kwargs) -> VisualStudio:
        """Verify that Visual Studio is available.

        :param tools: ToolCache of available tools
        :returns: A Visual Studio tool wrapper. Raises an exception if Visual Studio is
            not available.
        """
        # short circuit since already verified and available
        if hasattr(tools, "visualstudio"):
            return tools.visualstudio

        visualstudio = None

        try:
            # Try running MSBuild, assuming it is on the PATH.
            tools.subprocess.check_output(["MSBuild.exe", "--version"])
        except FileNotFoundError:
            # MSBuild isn't on the path
            pass
        except subprocess.CalledProcessError as e:
            raise BriefcaseCommandError(
                "MSBuild is on the path, but Briefcase cannot start it."
            ) from e
        else:
            # Create an explicit VisualStudio, with no install metadata
            visualstudio = VisualStudio(tools=tools, msbuild_path=Path("MSBuild.exe"))

        # try to find Visual Studio
        if visualstudio is None:
            # Look for an %MSBUILD% environment variable
            try:
                msbuild_path = Path(tools.os.environ["MSBUILD"])
                install_metadata: dict[str, str | int | bool] = None

                if not msbuild_path.exists():
                    # The location referenced by %MSBUILD% doesn't exist
                    raise BriefcaseCommandError(
                        f"""\
The location referenced by the environment variable MSBUILD:

    {msbuild_path}

does not appear to point to a MSBuild executable. Correct
or unset the environment variable; then re-run Briefcase.

"""
                    )

            except KeyError:
                # No %MSBUILD% environment variable. Look for vswhere.exe
                vswhere_path = None
                if program_files := tools.os.environ.get("ProgramFiles(x86)"):
                    vswhere_path = (
                        Path(program_files)
                        / "Microsoft Visual Studio"
                        / "Installer"
                        / "vswhere.exe"
                    )
                if vswhere_path is None or not vswhere_path.exists():
                    raise BriefcaseCommandError(
                        f"""\
Visual Studio does not appear to be installed. Visual Studio 2022 Community
Edition can be obtained as a free download from:

    https://visualstudio.microsoft.com/vs/community/

When you install Visual Studio, ensure you install the following workloads
and additional components:
{cls.VSCODE_REQUIRED_COMPONENTS}
If you have Visual Studio installed in a non-default location, you must
either put MSBuild.exe in your path, or define an MSBUILD environment
variable that points at the MSBuild.exe provided by your Visual Studio
installation.

"""
                    )

                # Retrieve metadata for Visual Studio install
                try:
                    install_metadata = tools.subprocess.parse_output(
                        json_parser,
                        [
                            vswhere_path,
                            "-latest",
                            "-prerelease",
                            "-format",
                            "json",
                        ],
                        stderr=subprocess.STDOUT,
                    )[0]
                except (
                    IndexError,
                    KeyError,
                    CommandOutputParseError,
                    subprocess.CalledProcessError,
                    OSError,
                ) as e:
                    raise BriefcaseCommandError(
                        f"""\
Visual Studio appears to exist, but Briefcase can't retrieve installation metadata.
Please report this as a bug at:

    https://github.com/beeware/briefcase/issues/new

In your report, please including the output from running:

    {vswhere_path}

from the command prompt.

"""
                    ) from e

                msbuild_path = (
                    Path(install_metadata["installationPath"])
                    / "MSBuild"
                    / "Current"
                    / "Bin"
                    / "MSBuild.exe"
                )
                if not msbuild_path.exists():
                    raise BriefcaseCommandError(
                        f"""\
Your Visual Studio installation does not appear to provide MSBuild.
Ensure that Visual Studio following workloads and components installed:
{cls.VSCODE_REQUIRED_COMPONENTS}
Then restart Briefcase.
"""
                    )

            # Try to invoke MSBuild at the established location
            try:
                tools.subprocess.check_output([msbuild_path, "--version"])
            except (subprocess.CalledProcessError, OSError):
                raise BriefcaseCommandError(
                    "MSBuild appears to exist, but Briefcase can't start it."
                )

            visualstudio = VisualStudio(
                tools=tools,
                msbuild_path=msbuild_path,
                install_metadata=install_metadata,
            )

        tools.visualstudio = visualstudio
        return visualstudio



import os
import sys
from unittest.mock import MagicMock

import pytest

from briefcase.console import Console, Log
from briefcase.integrations.subprocess import Subprocess
from briefcase.platforms.linux.flatpak import LinuxFlatpakOpenCommand

from ....utils import create_file


@pytest.fixture
def open_command(tmp_path, first_app_config):
    command = LinuxFlatpakOpenCommand(
        logger=Log(),
        console=Console(),
        base_path=tmp_path / "base_path",
        data_path=tmp_path / "briefcase",
    )
    command.tools.os = MagicMock(spec_set=os)
    command.tools.subprocess = MagicMock(spec_set=Subprocess)

    # Mock the call to verify the existence of the flatpak tools
    command.tools.subprocess.check_output.side_effect = [
        # flatpak --version
        "1.2.3",
        # flatpak-builder --version
        "1.2.3",
    ]

    return command


@pytest.mark.skipif(sys.platform != "linux", reason="Linux specific test")
def test_open(open_command, first_app_config, tmp_path):
    """On Linux, Open run `xdg-open` on the Content folder."""
    # Create the flatpak manifest file that would be in the project bundle.
    create_file(
        open_command.project_path(first_app_config) / "manifest.yml",
        "Flatpak manifest",
    )

    open_command(first_app_config)

    open_command.tools.subprocess.Popen.assert_called_once_with(
        [
            "xdg-open",
            tmp_path / "base_path/build/first-app/linux/flatpak",
        ]
    )



from unittest.mock import call

import pytest

from briefcase.console import InputDisabled
from tests.utils import default_rich_prompt


@pytest.mark.parametrize(
    "value, expected, default, transform",
    [
        ("A", "A", None, None),
        ("C", "C", None, None),
        ("", "D", "D", None),
        ("c", "C", None, str.upper),
    ],
)
def test_selection_input(console, value, expected, default, transform):
    prompt = "> "
    options = ["A", "B", "C", "D", "E", "F"]

    console.input.side_effect = [value]

    actual = console.selection_input(
        prompt=prompt,
        choices=options,
        default=default,
        transform=transform,
    )

    assert actual == expected
    console.input.assert_called_once_with(default_rich_prompt(prompt), markup=True)


def test_bad_input(console):
    """If the user enters bad input, they are shown an error and reprompted."""
    prompt = "> "
    options = ["A", "B", "C", "D", "E", "F"]

    console.input.side_effect = ["G", "Q", "C"]

    actual = console.selection_input(prompt=prompt, choices=options)

    assert actual == "C"
    assert console.input.call_count == 3
    assert console.input.call_args_list[0] == call(
        default_rich_prompt(prompt), markup=True
    )
    assert console.input.call_args_list[1] == call(
        default_rich_prompt(prompt), markup=True
    )
    assert console.input.call_args_list[2] == call(
        default_rich_prompt(prompt), markup=True
    )


def test_disabled(disabled_console):
    prompt = "> "
    options = ["A", "B", "C", "D", "E", "F"]

    actual = disabled_console.selection_input(
        prompt=prompt, choices=options, default="C"
    )

    assert actual == "C"
    disabled_console.input.assert_not_called()


def test_disabled_no_default(disabled_console):
    prompt = "> "
    options = ["A", "B", "C", "D", "E", "F"]

    with pytest.raises(InputDisabled):
        disabled_console.selection_input(
            prompt=prompt,
            choices=options,
            default=None,
        )

    disabled_console.input.assert_not_called()



import subprocess
from unittest.mock import Mock


def test_pidof_succeed(adb):
    """adb.pidof() can be called on a process that exists."""
    adb.run = Mock(return_value="5678\n")

    assert adb.pidof("com.example") == "5678"
    adb.run.assert_called_once_with("shell", "pidof", "-s", "com.example")


def test_pidof_quiet(adb):
    """adb.pidof() can be called in quiet mode on a process that exists."""
    adb.run = Mock(return_value="5678\n")

    assert adb.pidof("com.example", quiet=True) == "5678"
    adb.run.assert_called_once_with("shell", "pidof", "-s", "com.example", quiet=True)


def test_pidof_fail_exit_0(adb):
    """If adb.pidof() returns a PID of 0, it is interpreted as the process not
    existing."""
    adb.run = Mock(return_value="")

    assert adb.pidof("com.example") is None
    adb.run.assert_called_once_with("shell", "pidof", "-s", "com.example")


def test_pidof_fail_exit_1(adb):
    """If adb.pidof() fails, it is interpreted as the process not existing."""
    adb.run = Mock(side_effect=subprocess.CalledProcessError(1, "adb shell"))

    assert adb.pidof("com.example") is None
    adb.run.assert_called_once_with("shell", "pidof", "-s", "com.example")



import pytest

from briefcase.exceptions import UnsupportedHostError
from briefcase.integrations.file import File


def test_short_circuit(mock_tools):
    """Tool is not created if already cached."""
    mock_tools.file = "tool"

    tool = File.verify(mock_tools)

    assert tool == "tool"
    assert tool == mock_tools.file


def test_unsupported_os(mock_tools):
    """When host OS is not supported, an error is raised."""
    mock_tools.host_os = "wonky"

    # Delete download since it has already been verified
    delattr(mock_tools, "file")

    with pytest.raises(
        UnsupportedHostError,
        match=f"{File.name} is not supported on wonky",
    ):
        File.verify(mock_tools)


def test_verify(mock_tools):
    """Verifying Download always returns/set download tool."""
    file_tool = File.verify(mock_tools)
    assert isinstance(file_tool, File)
    assert file_tool is mock_tools.file



from __future__ import annotations

import os
import subprocess
import sys
from pathlib import Path

from briefcase.commands.run import RunAppMixin
from briefcase.config import AppConfig
from briefcase.exceptions import BriefcaseCommandError, RequirementsInstallError

from .base import BaseCommand
from .create import write_dist_info


class DevCommand(RunAppMixin, BaseCommand):
    cmd_line = "briefcase dev"
    command = "dev"
    output_format = None
    description = "Run a Briefcase project in the dev environment."

    # On macOS CoreFoundation/NSApplication will do its own independent parsing of argc/argv.
    # This means that whatever we pass to the Python interpreter on start-up will also be
    # visible to NSApplication which will interpret things like `-u` (used to make I/O
    # unbuffered in CPython) as `-u [URL]` (a request to open a document by URL). This is,
    # rather patently, Not Good.
    # To avoid this causing unwanted hilarity, we use environment variables to configure the
    # Python interpreter rather than command-line options.
    DEV_ENVIRONMENT = {
        # Equivalent of passing "-u"
        "PYTHONUNBUFFERED": "1",
        # Equivalent of passing "-X dev"
        "PYTHONDEVMODE": "1",
        # Equivalent of passing "-X utf8"
        "PYTHONUTF8": "1",
    }

    @property
    def platform(self):
        """The dev command always reports as the local platform."""
        return {
            "darwin": "macOS",
            "linux": "linux",
            "win32": "windows",
        }[sys.platform]

    def bundle_path(self, app):
        """A placeholder; Dev command doesn't have a bundle path."""
        raise NotImplementedError()

    def binary_path(self, app):
        """A placeholder; Dev command doesn't have a binary path."""
        raise NotImplementedError()

    def add_options(self, parser):
        parser.add_argument("-a", "--app", dest="appname", help="The app to run")
        parser.add_argument(
            "-r",
            "--update-requirements",
            action="store_true",
            help="Update requirements for the app",
        )
        parser.add_argument(
            "--no-run",
            dest="run_app",
            action="store_false",
            default=True,
            help="Do not run the app, just install requirements",
        )
        parser.add_argument(
            "--test",
            dest="test_mode",
            action="store_true",
            help="Run the app in test mode",
        )

    def install_dev_requirements(self, app: AppConfig, **options):
        """Install the requirements for the app dev.

        This will always include test requirements, if specified.

        :param app: The config object for the app
        """
        requires = app.requires if app.requires else []
        if app.test_requires:
            requires.extend(app.test_requires)

        if requires:
            with self.input.wait_bar("Installing dev requirements..."):
                try:
                    self.tools.subprocess.run(
                        [
                            sys.executable,
                            "-u",
                            "-X",
                            "utf8",
                            "-m",
                            "pip",
                            "install",
                            "--upgrade",
                        ]
                        + (["-vv"] if self.logger.is_deep_debug else [])
                        + requires,
                        check=True,
                        encoding="UTF-8",
                    )
                except subprocess.CalledProcessError as e:
                    raise RequirementsInstallError() from e
        else:
            self.logger.info("No application requirements.")

    def run_dev_app(
        self,
        app: AppConfig,
        env: dict,
        test_mode: bool,
        passthrough: list[str],
        **options,
    ):
        """Run the app in the dev environment.

        :param app: The config object for the app
        :param env: environment dictionary for sub command
        :param test_mode: Run the test suite, rather than the app?
        :param passthrough: A list of arguments to pass to the app
        """
        main_module = app.main_module(test_mode)

        # Add in the environment settings to get Python in the state we want.
        env.update(self.DEV_ENVIRONMENT)

        cmdline = [
            # Do not add additional switches for sys.executable; see DEV_ENVIRONMENT
            sys.executable,
            "-c",
            (
                "import runpy, sys;"
                "sys.path.pop(0);"
                f"sys.argv.extend({passthrough!r});"
                f'runpy.run_module("{main_module}", run_name="__main__", alter_sys=True)'
            ),
        ]

        # Console apps must operate in non-streaming mode so that console input can
        # be handled correctly. However, if we're in test mode, we *must* stream so
        # that we can see the test exit sentinel
        if app.console_app and not test_mode:
            self.logger.info("=" * 75)
            self.tools.subprocess.run(
                cmdline,
                env=env,
                encoding="UTF-8",
                cwd=self.tools.home_path,
                bufsize=1,
                stream_output=False,
            )
        else:
            app_popen = self.tools.subprocess.Popen(
                cmdline,
                env=env,
                encoding="UTF-8",
                cwd=self.tools.home_path,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                bufsize=1,
            )

            # Start streaming logs for the app.
            self._stream_app_logs(
                app,
                popen=app_popen,
                test_mode=test_mode,
                clean_output=False,
            )

    def get_environment(self, app, test_mode: bool):
        # Create a shell environment where PYTHONPATH points to the source
        # directories described by the app config.
        env = {
            "PYTHONPATH": os.pathsep.join(
                os.fsdecode(Path.cwd() / path) for path in app.PYTHONPATH(test_mode)
            )
        }

        # On Windows, we need to disable the debug allocator because it
        # conflicts with Python.net. See
        # https://github.com/pythonnet/pythonnet/issues/1977 for details.
        if self.platform == "windows":  # pragma: no branch
            env["PYTHONMALLOC"] = "default"  # pragma: no-cover-if-not-windows

        # If we're in verbose mode, put BRIEFCASE_DEBUG into the environment
        if self.logger.is_debug:
            env["BRIEFCASE_DEBUG"] = "1"

        return env

    def __call__(
        self,
        appname: str | None = None,
        update_requirements: bool | None = False,
        run_app: bool | None = True,
        test_mode: bool | None = False,
        passthrough: list[str] | None = None,
        **options,
    ):
        # Which app should we run? If there's only one defined
        # in pyproject.toml, then we can use it as a default;
        # otherwise look for a -a/--app option.
        if len(self.apps) == 1:
            app = list(self.apps.values())[0]
        elif appname:
            try:
                app = self.apps[appname]
            except KeyError as e:
                raise BriefcaseCommandError(
                    f"Project doesn't define an application named '{appname}'"
                ) from e

        else:
            raise BriefcaseCommandError(
                "Project specifies more than one application; use --app to specify which one to start."
            )
        # Confirm host compatibility, that all required tools are available,
        # and that the app configuration is finalized.
        self.finalize(app)

        self.verify_app(app)

        # Look for the existence of a dist-info file.
        # If one exists, assume that the requirements have already been
        # installed. If a dependency update has been manually requested,
        # do it regardless.
        dist_info_path = (
            self.app_module_path(app).parent / f"{app.module_name}.dist-info"
        )
        if not run_app:
            # If we are not running the app, it means we should update requirements.
            update_requirements = True
        if update_requirements or not dist_info_path.exists():
            self.logger.info("Installing requirements...", prefix=app.app_name)
            self.install_dev_requirements(app, **options)
            write_dist_info(app, dist_info_path)

        if run_app:
            if test_mode:
                self.logger.info(
                    "Running test suite in dev environment...", prefix=app.app_name
                )
            else:
                self.logger.info("Starting in dev mode...", prefix=app.app_name)
            env = self.get_environment(app, test_mode=test_mode)
            return self.run_dev_app(
                app,
                env,
                test_mode=test_mode,
                passthrough=[] if passthrough is None else passthrough,
                **options,
            )
